#!/usr/bin/env python
from __future__ import print_function
from test_python_common import *  # common subroutines

import cmor._cmor
import os

pth = os.path.split(os.path.realpath(os.curdir))
if pth[-1] == 'Test':
    ipth = opth = '.'
else:
    ipth = opth = 'Test'


myaxes = numpy.zeros(9, dtype='i')
myaxes2 = numpy.zeros(9, dtype='i')
myvars = numpy.zeros(9, dtype='i')


cmor.setup(
    inpath=ipth,
    set_verbosity=cmor.CMOR_NORMAL,
    netcdf_file_action=cmor.CMOR_REPLACE,
    exit_control=cmor.CMOR_EXIT_ON_MAJOR)
cmor.dataset(
    outpath=opth,
    experiment_id="historical",
    institution="GICC (Generic International Climate Center, Geneva, Switzerland)",
    source="GICCM1 2002: atmosphere:  GICAM3 (gicam_0_brnchT_itea_2, T63L32); ocean: MOM (mom3_ver_3.5.2, 2x3L15); sea ice: GISIM4; land: GILSM2.5",
    calendar="standard",
    realization=1,
    contact="Rusty Koder (koder@middle_earth.net)",
    history="Output from archive/giccm_03_std_2xCO2_2256.",
    comment="Equilibrium reached after 30-year spin-up after which data were output starting with nominal date of January 2030",
    references="Model described by Koder and Tolkien (J. Geophys. Res., 2001, 576-591).  Also see http://www.GICC.su/giccm/doc/index.html  2XCO2 simulation described in Dorkey et al. '(Clim. Dyn., 2003, 323-357.)",
    leap_year=0,
    leap_month=0,
    month_lengths=None,
    model_id="GICCM1",
    forcing="Ant, Nat",
    institute_id="pcmdi",
    parent_experiment_id="piControlwithism", branch_time=18336.33)

tables = []
a = cmor.load_table("/git/cmip5-cmor-tables/Tables/CMIP6_grids")
tables.append(a)

t = 'CMIP6_Omon.json'
te = 'dissic'
u = 'mol m-3'
time = 'time'
ts = 'month'
tscl = 1.

t = 'CMIP5_cfSites'
te = 'pr'
u = 'kg m-2 s-1'
time = 'time1'
ts = 'days'
tscl = 2.e-2

tables.append(cmor.load_table("/git/cmip5-cmor-tables/Tables/%s" % t))
print('Tables ids:', tables)

cmor.set_table(tables[0])

x, y, lon_coords, lat_coords, lon_vertices, lat_vertices = gen_irreg_grid(
    lon, lat)


myaxes[0] = cmor.axis(table_entry='y',
                      units='m',
                      coord_vals=y)
myaxes[1] = cmor.axis(table_entry='x',
                      units='m',
                      coord_vals=x)

grid_id = cmor.grid(axis_ids=myaxes[:2],
                    latitude=lat_coords,
                    longitude=lon_coords,
                    latitude_vertices=lat_vertices,
                    longitude_vertices=lon_vertices)
print('got grid_id:', grid_id)
myaxes[2] = grid_id

mapnm = 'lambert_conformal_conic'
params = ["standard_parallel1",
          "longitude_of_central_meridian", "latitude_of_projection_origin",
          "false_easting", "false_northing", "standard_parallel2"]
punits = ["", "", "", "", "", ""]
pvalues = [-20., 175., 13., 8., 0., 20.]
cmor.set_grid_mapping(grid_id=myaxes[2],
                      mapping_name=mapnm,
                      parameter_names=params,
                      parameter_values=pvalues,
                      parameter_units=punits)

cmor.set_table(tables[1])
myaxes[3] = cmor.axis(table_entry=time,
                      units='%s since 1980' % ts)

pass_axes = [myaxes[3], myaxes[2]]

myvars[0] = cmor.variable(table_entry=te,
                          units=u,
                          axis_ids=pass_axes,
                          history='no history',
                          comment='no future'
                          )

ntimes = 2
for i in range(ntimes):
    data2d = read_2d_input_files(i, varin2d[0], lat, lon) * 1.E-6
    print('writing time: ', i, data2d.shape)  # ,data2d
    # print Time[i],bnds_time[2*i:2*i+2]
    cmor.write(myvars[0], data2d, 1, time_vals=Time[i] * tscl,
               time_bnds=tscl * bnds_time[2 * i:2 * i + 2])
    print('wrote')
cmor.close()



#!/usr/bin/env python
import cmor
import numpy


def main():

    cmor.setup(inpath='Tables',
               netcdf_file_action=cmor.CMOR_REPLACE_3)
    cmor.dataset_json("Test/CMOR_input_example.json")

    table = 'CMIP6_Amon.json'
    cmor.load_table(table)
    axes = [{'table_entry': 'time',
             'units': 'days since 2000-01-01 00:00:00',
             },
            {'table_entry': 'latitude',
             'units': 'degrees_north',
             'coord_vals': [0],
             'cell_bounds': [-1, 1]},
            {'table_entry': 'longitude',
             'units': 'degrees_east',
             'coord_vals': [90],
             'cell_bounds': [89, 91]},
            ]

    values = numpy.array([1.], numpy.float32) + 200
    axis_ids = list()
    for axis in axes:
        axis_id = cmor.axis(**axis)
        axis_ids.append(axis_id)

    for var, units, positive in (('ts', 'K', ''),
                                 ('rsut', 'W m-2', 'up'),
                                 ('rlut', 'W m-2', 'down'),):
        varid = cmor.variable(var,
                              units,
                              axis_ids,
                              history='variable history',
                              missing_value=-99,
                              positive=positive
                              )
        cmor.write(varid, values, time_vals=[15], time_bnds=[[0, 30]])

    cmor.close()


if __name__ == '__main__':

    main()



# If this example is not executed from the directory containing the
# CMOR code, please first complete the following steps:
#
#   1. In any directory, create 'Tables/', 'Test/' and 'CMIP6/' directories.
#
#   2. Download
#      https://github.com/PCMDI/cmor/blob/master/TestTables/CMIP6_Omon.json
#      and https://github.com/PCMDI/cmor/blob/master/TestTables/CMIP6_CV.json
#      to the 'Tables/' directory.
#
#   3. Download
#      https://github.com/PCMDI/cmor/blob/master/Test/<filename>.json
#      to the 'Test/' directory.

import cmor
import numpy
import unittest
import sys
import os
import tempfile
import base_CMIP6_CV


def run():
    unittest.main()


class TestCase(base_CMIP6_CV.BaseCVsTest):

    def testCMIP6(self):
        try:
            # -------------------------------------------
            # Try to call cmor with a bad institution_ID
            # -------------------------------------------
            global testOK
            cmor.setup(inpath='Tables', netcdf_file_action=cmor.CMOR_REPLACE, logfile=self.tmpfile)
            cmor.dataset_json("Test/CMOR_input_example.json")
            cmor.set_cur_dataset_attribute("source_id", "invalid")

            # ------------------------------------------
            # load Omon table and create masso variable
            # ------------------------------------------
            cmor.load_table("CMIP6_Omon.json")
            itime = cmor.axis(table_entry="time", units='months since 2010',
                              coord_vals=numpy.array([0, 1, 2, 3, 4.]),
                              cell_bounds=numpy.array([0, 1, 2, 3, 4, 5.]))
            ivar = cmor.variable(
                table_entry="masso",
                axis_ids=[itime],
                units='kg')

            data = numpy.random.random(5)
            for i in range(0, 5):
                cmor.write(ivar, data[i:i])
            cmor.close()
        except BaseException:
            pass
        self.assertCV("invalid")

if __name__ == '__main__':
    run()



from __future__ import print_function
import cmor
import numpy


nlat = 3600
dlat = 180. / nlat
nlon = 7200
dlon = 360. / nlon
nlev = 26
dlev = 1000. / nlev
ntimes = 1

lats = numpy.arange(-90 + dlat / 2., 90, dlat)
blats = numpy.arange(-90, 90 + dlat, dlat)
lons = numpy.arange(0 + dlon / 2., 360., dlon)
blons = numpy.arange(0, 360. + dlon, dlon)

levs = numpy.array([1000., 925, 900, 850, 800, 700, 600, 500, 400, 300,
                    250, 200, 150, 100, 75, 70, 50, 30, 20, 10, 7.5, 5, 2.5, 1, .5, .1])
alllevs = numpy.arange(1000, 0, -dlev).tolist()
print(len(alllevs))

cmor.setup(inpath='Tables', netcdf_file_action=cmor.CMOR_REPLACE)
cmor.dataset_json("Test/CMOR_input_example.json")
table = 'CMIP6_Amon.json'
cmor.load_table(table)


ilat = cmor.axis(
    table_entry='latitude',
    coord_vals=lats,
    cell_bounds=blats,
    units='degrees_north')
ilon = cmor.axis(
    table_entry='longitude',
    coord_vals=lons,
    cell_bounds=blons,
    units='degrees_east')
# ,coord_vals=numpy.arange(ntimes,dtype=float),cell_bounds=numpy.arange(ntimes+1,dtype=float),units='months since 2000')
itim = cmor.axis(table_entry='time', units='months since 2010')
ilev = cmor.axis(table_entry='plev19', coord_vals=levs, units='hPa')

axes = [itim, ilev, ilat, ilon]

var = cmor.variable(table_entry='ta', units='K', axis_ids=axes)

print("allocating mem for data")
data = numpy.random.random((nlev, nlat, nlon)) * 30 + 273.15
print("moving on to writing")

for i in range(ntimes):
    print('Writing time:', i)
    cmor.write(var, data, time_vals=numpy.array(
        [float(i), ]), time_bnds=numpy.array([i, i + 1.]))

print("closing var")
print(cmor.close(var_id=var, file_name=True))
print("closing cmor")
cmor.close()
print("done")


print('hello')



import cmor
import numpy


def cmor_initialisation():
    cmor.setup(inpath='/git/cmip5-cmor-tables/Tables',
               netcdf_file_action=cmor.CMOR_REPLACE_3,
               create_subdirectories=0)
    cmor.dataset('pre-industrial control', 'ukmo', 'HadCM3', '360_day',
                 institute_id='ukmo',
                 model_id='HadCM3',
                 history='some global history',
                 forcing='N/A',
                 parent_experiment_id='N/A',
                 parent_experiment_rip='N/A',
                 branch_time=0.,
                 contact='bob',
                 outpath='hadju/test')


def setup_data():
    axes = [{'table_entry': 'time',
             'units': 'days since 2000-01-01 00:00:00'
             },
            {'table_entry': 'latitude',
             'units': 'degrees',
             'coord_vals': [0],
             'cell_bounds': [-0.5, 0.5]},
            {'table_entry': 'longitude',
             'units': 'degrees',
             'coord_vals': [1],
             'cell_bounds': [0.5, 1.5]},
            ]

    values = numpy.array([215.], numpy.float32)
    return values, axes


def cmor_define_and_write(values, axes):
    table = 'CMIP5_day'
    cmor.load_table(table)

    axis_ids = list()
    for axis in axes:
        axis_ids.append(cmor.axis(**axis))

    varid = cmor.variable('rlut',
                          'W m-2',
                          axis_ids,
                          history='variable history',
                          missing_value=-99,
                          positive='up'
                          )

    for time in (15, 16, 17):
        cmor.write(
            varid,
            values,
            time_vals=[time],
            time_bnds=[
                time -
                0.45,
                time +
                0.45])


def version(cmor):
    return '%s.%s.%s' % (cmor.CMOR_VERSION_MAJOR,
                         cmor.CMOR_VERSION_MINOR,
                         cmor.CMOR_VERSION_PATCH)


def main():
    assert version(cmor) == '2.8.3'
    cmor_initialisation()
    values, axes = setup_data()
    cmor_define_and_write(values, axes)
    cmor.close()


if __name__ == '__main__':

    main()



from __future__ import print_function
import cmor
import numpy

nlat = 10
dlat = 180. / nlat
nlon = 20
dlon = 360. / nlon
nlev = 17
ntimes = 1

lats = numpy.arange(90 - dlat / 2., -90, -dlat)
blats = numpy.arange(90, -90 - dlat, -dlat)
lats2 = numpy.arange(-90 + dlat / 2., 90, dlat)
blats2 = numpy.arange(-90, 90 + dlat, dlat)
lons = numpy.arange(0 + dlon / 2., 360., dlon)
blons = numpy.arange(0, 360. + dlon, dlon)

cmor.setup(inpath='.', netcdf_file_action=cmor.CMOR_REPLACE)
cmor.dataset_json("Test/CMOR_input_example.json")
table = 'Tables/CMIP6_Amon.json'
cmor.load_table(table)

data = lats[:, numpy.newaxis] * lons[numpy.newaxis, :]

data = (data + 29000) / 750. + 233.2


ilat = cmor.axis(
    table_entry='latitude',
    coord_vals=lats,
    cell_bounds=blats,
    units='degrees_north')
ilat2 = cmor.axis(
    table_entry='latitude',
    coord_vals=lats2,
    cell_bounds=blats2,
    units='degrees_north')
ilon = cmor.axis(
    table_entry='longitude',
    coord_vals=lons,
    cell_bounds=blons,
    units='degrees_east')
itim = cmor.axis(table_entry='time', units='months since 2010')


ivar = cmor.variable(
    table_entry='tasmin',
    units='K',
    axis_ids=[
        itim,
        ilat,
        ilon])
cmor.write(ivar, data, ntimes_passed=1, time_vals=[0.], time_bnds=[0., 1.])
fnm = cmor.close(ivar, file_name=True)

print('*******************************')

ivar2 = cmor.variable(
    table_entry='tasmin',
    units='K',
    axis_ids=[
        itim,
        ilat2,
        ilon])
cmor.write(ivar2, data, ntimes_passed=1, time_vals=[1.], time_bnds=[1., 2.])
fnm2 = cmor.close(ivar2, file_name=True)

## import cdms2,vcs
# x=vcs.init()
# x.portrait()
## import EzTemplate
# M=EzTemplate.Multi(columns=1,rows=3)
# print 'Getting templates'
# t1=M.get()
# t2=M.get()
# print 'Plotting'
# x.plot(data,t1)
# f=cdms2.open(fnm)
# s=f("tasmin")
# print 'Ok data read, shape:',s.shape
# x.plot(s,t2)
# f=cdms2.open(fnm2)
# s=f("tasmin")
# print 'Ok data read, shape:',s.shape
# x.plot(s,M.get())

# raw_input()



import cmor
import numpy
import unittest
import sys
import os
import tempfile
import re

class TestCase(unittest.TestCase):

    def setUp(self, *args, **kwargs):
        # create temporary log file
        self.logfile = tempfile.mkstemp()[1]

    def tearDown(self):
        os.unlink(self.logfile)

    def testNotEnoughTimesWritten(self):
        try:

            time_len = 5
            time_vals = numpy.arange(time_len)
            time_bnds = numpy.arange(time_len+1)

            cmor.setup(inpath='Tables', netcdf_file_action=cmor.CMOR_REPLACE, logfile=self.logfile)
            cmor.dataset_json("Test/CMOR_input_example.json")
            cmor.set_cur_dataset_attribute(
                "source_type", "AOGCM ISM")

            cmor.load_table("CMIP6_Omon.json")
            itime = cmor.axis(table_entry="time", units='months since 2010',
                              coord_vals=time_vals,
                              cell_bounds=time_bnds)
            ivar = cmor.variable(
                table_entry="masso",
                axis_ids=[itime],
                units='kg')

            data_len = time_len - 1
            data = numpy.random.random(data_len)
            for i in range(0, data_len):
                a = cmor.write(ivar, data[i:i+1], ntimes_passed=1)
                
            cmor.close()

            # find "not enough times written" warning in log
            warn_msg = "! Warning: while closing variable {} (masso, table Omon)\n" \
                       "! we noticed you wrote {} time steps for the variable,\n" \
                       "! but its time axis {} (time) has {} time steps" \
                        .format(ivar, data_len, itime, time_len)
            remove_ansi = re.compile(r'\x1b\[[0-?]*[ -/]*[@-~]')
            with open(self.logfile) as f:
                lines = f.readlines()
                log_text = remove_ansi.sub('', ''.join(lines))
                self.assertIn(warn_msg, log_text)

        except BaseException:
            raise


if __name__ == '__main__':
    unittest.main()



import os
import re
import sys
import time
import argparse

parser = argparse.ArgumentParser(
    description='Rebuild meta.yaml from conda-forge feedstock',
    formatter_class=argparse.ArgumentDefaultsHelpFormatter)

parser.add_argument("-p", "--package_name",
                    help="Package name to build")
parser.add_argument("-o", "--organization",
                    help="github organization name", default="PCMDI")
parser.add_argument("-r", "--repo_name",
                    help="repo name to build", default="cmor")
parser.add_argument("-b", "--branch", default='main', help="branch to build")
parser.add_argument("-v", "--version",
                    help="version are we building for")
parser.add_argument("-g", "--git_rev",
                    help="Latest commit")
parser.add_argument("-B", "--build", default="0",
                    help="build number, this should be 0 for nightly")
parser.add_argument("--local_repo", help="Path to local project repository")
parser.add_argument("--src_meta_yaml", help="Path to source meta.yaml")
parser.add_argument("--dst_meta_yaml", help="Path to destination meta.yaml")

args = parser.parse_args(sys.argv[1:])

print("Rebuilding meta.yaml with the following...")
for k, v in vars(args).items():
    print(f"{k}: {v}")

repo_url = f"https://github.com/{args.organization}/{args.repo_name}.git"

package_time = time.strftime("%Y.%m.%d.%H.%M", time.localtime())
version = f"{args.version}.{package_time}.{args.git_rev}"

with open(args.src_meta_yaml, "r") as src, \
        open(args.dst_meta_yaml, "w") as dst:
    start_copy = True
    lines = src.readlines()
    for line in lines:
        match_obj = re.match("package:", line)
        if match_obj:
            start_copy = False
            dst.write("package:\n")
            dst.write(f"  name: {args.package_name}\n")
            dst.write(f"  version: {version}\n\n")

            dst.write("source:\n")
            if args.local_repo is None:
                dst.write(f"  git_rev: {args.branch}\n")
                dst.write(f"  git_url: {repo_url}\n\n")
            else:
                dst.write(f"  path: {args.local_repo}\n\n")

            continue

        match_obj = re.match("build:", line)
        if match_obj:
            start_copy = True

        match_build_number = re.match(r"\s+number:", line)
        if match_build_number:
            dst.write(f"  number: {args.build}\n")
            continue
        if start_copy:
            dst.write(line)
        else:
            continue



import cmor
import numpy

error_flag = cmor.setup(inpath='Tables', netcdf_file_action=cmor.CMOR_REPLACE)

error_flag = cmor.dataset_json("Test/CMOR_input_example.json")

cmor.load_table("CMIP6_Omon.json")
itime = cmor.axis(table_entry="time", units='months since 2010', coord_vals=numpy.array(
    [0, 1, 2, 3, 4.]), cell_bounds=numpy.array([0, 1, 2, 3, 4, 5.]))
ivar = cmor.variable(table_entry="masso", axis_ids=[itime], units='kg')

data = numpy.random.random(5)
for i in range(0, 5):
    # ,time_vals=numpy.array([i,]),time_bnds=numpy.array([i,i+1]))
    cmor.write(ivar, data[i:i])
error_flag = cmor.close()



from __future__ import print_function
import cmor
import numpy

ipth = "Test"
cmor.setup(inpath=ipth,
           set_verbosity=cmor.CMOR_NORMAL,
           netcdf_file_action=cmor.CMOR_REPLACE,
           logfile=None)

cmor.dataset(
    outpath=ipth,
    experiment_id="lgm",
    institution="PCMDI",
    source="PCMDI",
    calendar="standard",
    model_id="pcmdi-09a", forcing="forcing")

cmor.load_table("Tables/CMIP6_Omon.json")

nlat = 90
dlat = 180 / nlat
nlon = 180
dlon = 360. / nlon
nlev = 5

lats = numpy.arange(-90 + dlat / 2., 90, dlat)
blats = numpy.arange(-90, 90 + dlat, dlat)
lons = numpy.arange(0 + dlon / 2., 360., dlon)
blons = numpy.arange(0, 360. + dlon, dlon)

ntime = 12

data = numpy.random.random((ntime, nlat, nlev, nlon)) * 5 + 273.

itim = cmor.axis(
    table_entry='time', coord_vals=numpy.arange(
        0, ntime, 1), units='month since 2008', cell_bounds=numpy.arange(
            0, ntime + 1, 1))
ilat = cmor.axis(
    table_entry='latitude',
    coord_vals=lats,
    units='degrees_north',
    cell_bounds=blats)
ilon = cmor.axis(
    table_entry='longitude',
    coord_vals=lons,
    units='degrees_east',
    cell_bounds=blons)
ilev = cmor.axis(table_entry='depth_coord',
                 length=5,
                 cell_bounds=numpy.arange(-12000,
                                          0,
                                          2000),
                 coord_vals=numpy.arange(-10000,
                                         0,
                                         2000),
                 units="m")

iv = cmor.variable(
    table_entry='thetao', axis_ids=numpy.array(
        (itim, ilat, ilev, ilon)), units='K')

cmor.write(iv, data)

f1 = cmor.close(iv, file_name=True)
print(f1)




import cmor._cmor
import os
import unittest
import numpy
import cdms2


class TestForecastTime(unittest.TestCase):

    def setup(self):
        self.path = None

    def test_ReferenceTime(self):
        from test_python_common import Time, bnds_time, read_2d_input_files
        from test_python_common import alats, alons, bnds_lat, bnds_lon
        from test_python_common import entry2d, units2d, positive2d
        from test_python_common import varin2d, ntimes, lat, lon
        dtmp2 = 1.e-4

        myaxes = numpy.zeros(9, dtype='i')
        myvars = numpy.zeros(9, dtype='i')

        cmor.setup(
            inpath="Tables",
            set_verbosity=cmor.CMOR_NORMAL,
            netcdf_file_action=cmor.CMOR_REPLACE,
            exit_control=cmor.CMOR_EXIT_ON_MAJOR)

        cmor.dataset_json("Test/CMOR_input_example.json")

        cmor.load_table(os.path.join("CMIP6_Amon.json"))

        # ok we need to make the bounds 2D because the cmor module "undoes
        # this"
        bnds_time = numpy.reshape(bnds_time, (bnds_time.shape[0] / 2, 2))
        bnds_lat = numpy.reshape(bnds_lat, (bnds_lat.shape[0] / 2, 2))
        bnds_lon = numpy.reshape(bnds_lon, (bnds_lon.shape[0] / 2, 2))

        id = 'time'
        units = 'months since 1980'
        myaxes[0] = cmor.axis(
            id,
            coord_vals=Time,
            units=units,
            cell_bounds=bnds_time,
            interval="1 month")

        id = 'latitude'
        units = "degrees_north"
        myaxes[1] = cmor.axis(
            id,
            coord_vals=alats,
            units=units,
            cell_bounds=bnds_lat)

        id = "longitude"
        units = "degrees_east"
        myaxes[2] = cmor.axis(
            id,
            coord_vals=alons,
            units=units,
            cell_bounds=bnds_lon)

        id = 'reftime'
        units = 'months since 1980'
        myaxes[3] = cmor.axis(
            id,
            coord_vals=[20],
            units=units)

        myvars[0] = cmor.variable(entry2d[0],
                                  units2d[0],
                                  myaxes[:4],
                                  'd',
                                  missing_value=None,
                                  tolerance=dtmp2,
                                  positive=positive2d[0],
                                  original_name=varin2d[0],
                                  history="no history",
                                  comment="no future")

        #  /* ok here we decalre a variable for region axis testing */
        for i in range(ntimes):
            data2d = read_2d_input_files(i, varin2d[0], lat, lon)
            cmor.write(myvars[0], data2d, ntimes_passed=1)

        self.path = cmor.close(var_id=0, file_name=True)

        f = cdms2.open(self.path)
        self.assertIn('reftime', f.listdimension())
        data = f['reftime']
        self.assertEqual(data[0], 600)

    def tearDown(self):
        if self.path:
            if os.path.exists(self.path):
                os.remove(self.path)


if __name__ == '__main__':
    unittest.main()



from __future__ import print_function
import os
import sys

svn = sys.argv[1]

git = os.popen("git status").readlines()

modfiles = []
delfiles = []
newfiles = []
for l in git:
    if l.find("new file:") > -1:
        newfiles.append(l.split()[3])
    if l.find("modified:") > -1:
        modfiles.append(l.split()[2])
    if l.find("deleted:") > -1:
        delfiles.append(l.split()[2])

for f in modfiles + newfiles:
    cmd = "cp -pf %s %s/%s" % (f, svn, f)
    print('Cp:', cmd)
    os.popen(cmd).readlines()

for f in newfiles:
    cmd = "cd %s ; svn add %s" % (svn, f)
    print('svn add :', cmd)
    os.popen(cmd).readlines()

for f in delfiles:
    cmd = "cd %s ; svn delete --force %s" % (svn, f)
    print('svn del :', cmd)
    os.popen(cmd).readlines()



# If this example is not executed from the directory containing the
# CMOR code, please first complete the following steps:
#
#   1. In any directory, create 'Tables/', 'Test/' and 'CMIP6/' directories.
#
#   2. Download
#      https://github.com/PCMDI/cmor/blob/master/TestTables/CMIP6_Omon.json
#      and https://github.com/PCMDI/cmor/blob/master/TestTables/CMIP6_CV.json
#      to the 'Tables/' directory.
#
#   3. Download
#      https://github.com/PCMDI/cmor/blob/master/Test/<filename>.json
#      to the 'Test/' directory.

import cmor
import numpy
import unittest
import os
import sys
import tempfile


# ==============================
#  main thread
# ==============================


def run():
    unittest.main()


class TestCase(unittest.TestCase):

    def setUp(self, *args, **kwargs):
        # ------------------------------------------------------
        # Copy stdout and stderr file descriptor for cmor output
        # ------------------------------------------------------
        self.newstdout = os.dup(1)
        self.newstderr = os.dup(2)
        # --------------
        # Create tmpfile
        # --------------
        self.tmpfile = tempfile.mkstemp()
#        os.dup2(self.tmpfile[0], 1)
#        os.dup2(self.tmpfile[0], 2)
        os.close(self.tmpfile[0])

    def getAssertTest(self):
        f = open(self.tmpfile[1], 'r')
        lines = f.readlines()
        for line in lines:
            if line.find('Error:') != -1:
                testOK = line.strip()
                break
        f.close()
        os.unlink(self.tmpfile[1])
        return testOK

    def testCMIP6(self):
        try:

            cmor.setup(inpath='Tables', netcdf_file_action=cmor.CMOR_REPLACE)
            cmor.dataset_json("Test/CMOR_input_example.json")

            cmor.load_table("CMIP6_Omon.json")
            itime = cmor.axis(table_entry="time", units='months since 2010', coord_vals=numpy.array(
                [0, 1, 2, 3, 4.]), cell_bounds=numpy.array([0, 1, 2, 3, 4, 5.]))
            ivar = cmor.variable(
                table_entry="masso",
                axis_ids=[itime],
                units='kg')

            data = numpy.random.random(5)
            for i in range(0, 5):
                # ,time_vals=numpy.array([i,]),time_bnds=numpy.array([i,i+1]))
                cmor.write(ivar, data[i:i])
            cmor.close()
        except BaseException:
            os.dup2(self.newstdout, 1)
            os.dup2(self.newstderr, 2)
            sys.stdout = os.fdopen(self.newstdout, 'w', 0)
            sys.stderr = os.fdopen(self.newstderr, 'w', 0)
            self.assertIn('d', testOK)


if __name__ == '__main__':
    run()



import cmor
import numpy
import unittest
import sys
import os
import tempfile


class TestCase(unittest.TestCase):

    # This test demonstrates an exception that gets thrown when
    # ntimes_passed is greater than the number of times passed in data
    def testNotEnoughData(self):

        cmor.setup(inpath='Tables',
                netcdf_file_action=cmor.CMOR_REPLACE)
        cmor.dataset_json("Test/CMOR_input_example.json")

        table = 'CMIP6_Omon.json'
        cmor.load_table(table)
        axes = [{'table_entry': 'time',
                'units': 'days since 1850-01-01 00:00:00',
                'coord_vals': [15.5, 45, ],
                'cell_bounds':[[0, 31], [31, 62]]
                },
                {'table_entry': 'depth_coord_half',
                'units': 'm',
                'coord_vals': [5000., 3000., 2000., 1000.],
                'cell_bounds': [5000., 3000., 2000., 1000., 0]},
                {'table_entry': 'latitude',
                'units': 'degrees_north',
                'coord_vals': [0],
                'cell_bounds': [-1, 1]},
                {'table_entry': 'longitude',
                'units': 'degrees_east',
                'coord_vals': [90],
                'cell_bounds': [89, 91]},
                ]

        axis_ids = list()
        for axis in axes:
            axis_id = cmor.axis(**axis)
            axis_ids.append(axis_id)

        for var, units, value in (('zhalfo', 'm', 274.),):
            values = numpy.ones([len(x['coord_vals']) for x in axes]) * value
            values = values.astype("f")
            varid = cmor.variable(var,
                                units,
                                axis_ids,
                                history='variable history',
                                missing_value=-99
                                )
            try:
                cmor.write(varid, values, ntimes_passed=3)
            except Exception as inst:
                self.assertEqual(str(inst), "not enough data is being passed "
                                            "for the number of times passed")
            else:
                raise Exception("cmor.write did not throw expected exception")

        cmor.close()

if __name__ == '__main__':
    unittest.main()



input_tables = {
    'climatology_test_code': ['climatology_test_table_A'],
    'test_any_from_asc': ['CMOR_SAMPLE_TABLE'],
    'test_any_from_asc_d': ['CMOR_SAMPLE_TABLE'],
    'test_any_from_asc_i': ['CMOR_SAMPLE_TABLE'],
    'karls_test': ['IPCC_test_table_A'],
    'test1': ['CMOR_SAMPLE_TABLE'],
    'test2': ['CMOR_SAMPLE_TABLE'],
    'test3': ['CMOR_SAMPLE_TABLE'],
    'test4': ['CMOR_SAMPLE_TABLE'],
    'main_prog': ['CMOR_SAMPLE_TABLE'],
    'test_any_from_asc_i': ['CMOR_SAMPLE_TABLE', ],
    'mytest_4d_r': ['CMOR_SAMPLE_TABLE'],
    'rewrite_harvardf_data': ['Curts_table'],
    'test_3h': ['IPCC_table_A3'],
    'test_dimensionless': ['IPCC_table_A1'],
    'test_fortran_example_00': ['IPCC_test_table_A'],
    'test_fortran_example_01': ['IPCC_test_table_A', 'IPCC_test_table_Grids'],
    'test_station_data': ['IPCC_test_table_S'],
    'test_region': ['IPCC_test_table_O'],
    'test_sigma': ['IPCC_table_A1'],
    'test_singleton': ['IPCC_test_table_A'],
    'mytest_4d_r_big_array': ['CMOR_SAMPLE_TABLE'],
    'mytest_4d_d_big_array_2': ['CMOR_SAMPLE_TABLE'],
    'mytest_4d_d_big_array_3': ['CMOR_SAMPLE_TABLE'],
    'mytest_4d_d_big_array_4': ['CMOR_SAMPLE_TABLE'],
    'mytest_4d_d_big_array_5': ['CMOR_SAMPLE_TABLE'],
    'test_lots_of_variables': ['IPCC_test_table_As'],
    'test_shrt_exp_nm_set_att_initi': ['IPCC_test_table_As'],
    'test_sophie': ['Sophie_Table'],
}



# If this example is not executed from the directory containing the
# CMOR code, please first complete the following steps:
#
#   1. In any directory, create 'Tables/', 'Test/' and 'CMIP6/' directories.
#
#   2. Download
#      https://github.com/PCMDI/cmor/blob/master/TestTables/CMIP6_Omon.json
#      and https://github.com/PCMDI/cmor/blob/master/TestTables/CMIP6_CV.json
#      to the 'Tables/' directory.
#
#   3. Download
#      https://github.com/PCMDI/cmor/blob/master/Test/<filename>.json
#      to the 'Test/' directory.

import cmor
import numpy
import unittest
import sys
import os
import tempfile
import pkgutil
import netCDF4
import base_CMIP6_CV


# ==============================
#  main thread
# ==============================
def run():
    unittest.main()


class TestCase(base_CMIP6_CV.BaseCVsTest):

    def testCMIP6(self):
        # -------------------------------------------
        # Try to call cmor with a bad institution_ID
        # -------------------------------------------
        cmor.setup(inpath='Tables', netcdf_file_action=cmor.CMOR_REPLACE, logfile=self.tmpfile)
        cmor.dataset_json("Test/CMOR_input_example.json")

        # ------------------------------------------
        # load Omon table and create masso variable
        # ------------------------------------------
        cmor.load_table("CMIP6_Omon.json")
        itime = cmor.axis(table_entry="time", units='months since 2010',
                            coord_vals=numpy.array([0, 1, 2, 3, 4.]),
                            cell_bounds=numpy.array([0, 1, 2, 3, 4, 5.]))
        ivar = cmor.variable(
            table_entry="masso",
            axis_ids=[itime],
            units='kg')

        data = numpy.random.random(5)
        for i in range(0, 5):
            cmor.write(ivar, data[i:i])
        self.delete_files += [cmor.close(ivar, True)]
        filen = cmor.close()
        f = netCDF4.Dataset(cmor.get_final_filename(), "r")
        self.assertTrue("tracking_id" in f.__dict__)
        a = f.__dict__["tracking_id"].split('/')[0]
        self.assertNotIn("hdl:21.14100/", a)

    @unittest.skipIf(pkgutil.find_loader("cdms2") is None, "cdms2 not installed")
    def testCMIP6_CDMS2(self):
        import cdms2

        # -------------------------------------------
        # Try to call cmor with a bad institution_ID
        # -------------------------------------------
        cmor.setup(inpath='Tables', netcdf_file_action=cmor.CMOR_REPLACE, logfile=self.tmpfile)
        cmor.dataset_json("Test/CMOR_input_example.json")

        # ------------------------------------------
        # load Omon table and create masso variable
        # ------------------------------------------
        cmor.load_table("CMIP6_Omon.json")
        itime = cmor.axis(table_entry="time", units='months since 2010',
                            coord_vals=numpy.array([0, 1, 2, 3, 4.]),
                            cell_bounds=numpy.array([0, 1, 2, 3, 4, 5.]))
        ivar = cmor.variable(
            table_entry="masso",
            axis_ids=[itime],
            units='kg')

        data = numpy.random.random(5)
        for i in range(0, 5):
            cmor.write(ivar, data[i:i])
        self.delete_files += [cmor.close(ivar, True)]
        filen = cmor.close()
        f = cdms2.open(cmor.get_final_filename(), "r")
        a = f.getglobal("tracking_id").split('/')[0]
        self.assertNotIn("hdl:21.14100/", a)


if __name__ == '__main__':
    run()



from cmip6_cv.cmor_const import *

from cmip6_cv.pywrapper import setup, load_table, set_table, close, set_cur_dataset_attribute, get_cur_dataset_attribute, has_cur_dataset_attribute, set_variable_attribute, list_variable_attributes, get_variable_attribute, has_variable_attribute, check_institution, check_sourceID, check_experiment, check_grids, check_requiredattributes, check_ISOTime, check_furtherinfourl, get_CV_Error, setup_variable, set_CV_Error, reset_CV_Error, check_parentExpID, check_subExpID, check_filename



from __future__ import print_function
import cmor

# Hypothetical data are going from April 2049 thru March 2050
ntimes = 12
times = [18015,
        18045,
        18075,
        18105,
        18135,
        18165,
        18195,
        18225,
        18255,
        18285,
        18315,
        18345]
# first full djf one year later than first full mam
times_bnds = [(0, 36030),
            (30, 36060),
            (60, 36090),
            (90, 36120),
            (120, 36150),
            (150, 36180),
            (180, 36210),
            (210, 36240),
            (240, 36270),
            (270, 36300),
            (300, 36330),
            (330, 36360)]

def path_test():
    cmor.setup(inpath='Tables', netcdf_file_action=cmor.CMOR_REPLACE)

    cmor.dataset_json("Test/CMOR_input_example.json")


    table = 'CMIP6_Amon.json'
    cmor.load_table(table)
    axes = [{'table_entry': 'time2',
             'units': 'days since 2000-01-01 00:00:00',
#             'coord_vals': times,
#             'cell_bounds': times_bnds,
             },
            {'table_entry': 'plev19',
             'units': 'Pa',
             'coord_vals': [100000., 92500., 85000., 70000., 60000., 50000.,
                            40000., 30000., 25000., 20000., 15000.,
                            10000., 7000., 5000., 3000., 2000., 1000., 500, 100]},
            {'table_entry': 'latitude',
             'units': 'degrees_north',
             'coord_vals': [0],
             'cell_bounds': [-1, 1]},
            {'table_entry': 'longitude',
             'units': 'degrees_east',
             'coord_vals': [90, 88],
             'cell_bounds': [ (91, 89), (89, 87)]},
            ]

    axis_ids = list()
    for axis in axes:
        axis_id = cmor.axis(**axis)
        axis_ids.append(axis_id)
    varid = cmor.variable('co2Clim', '1.e-6', axis_ids)
    import numpy
    data = numpy.arange(ntimes*19*2)
    data.resize((ntimes, 19, 1, 2))
#    cmor.write(varid, data)
#    for i in range(len(data)):
#        cmor.write(varid, data[i], time_vals=times[i], time_bnds=times_bnds[i])
    cmor.write(varid, data, time_vals=times, time_bnds=times_bnds)
    path = cmor.close(varid, file_name=True)

    print(path)


if __name__ == '__main__':
    path_test()



import cmor
import numpy
import unittest
import os
import shutil
import tempfile
from netCDF4 import Dataset


def run():
    unittest.main()


class TestCase(unittest.TestCase):

    def gen_file(self, seed, ntimes, zstd_level,
                 deflate, deflate_level, shuffle,
                 quantize_mode, quantize_nsd, out_dir):

        numpy.random.seed(seed)

        cmor.setup(inpath='Test', netcdf_file_action=cmor.CMOR_REPLACE)

        cmor.dataset_json("Test/CMOR_input_example.json")

        # creates 1 degree grid
        nlat = 18
        nlon = 36
        alats = numpy.arange(180) - 89.5
        bnds_lat = numpy.arange(181) - 90
        alons = numpy.arange(360) + .5
        bnds_lon = numpy.arange(361)
        cmor.load_table("Tables/CMIP6_Amon.json")
        ilat = cmor.axis(
            table_entry='latitude',
            units='degrees_north',
            length=nlat,
            coord_vals=alats,
            cell_bounds=bnds_lat)

        ilon = cmor.axis(
            table_entry='longitude',
            length=nlon,
            units='degrees_east',
            coord_vals=alons,
            cell_bounds=bnds_lon)

        plevs = numpy.array([100000., 92500, 85000, 70000, 60000, 50000, 40000,
                            30000, 25000, 20000, 15000, 10000, 7000, 5000, 3000,
                            2000, 1000, 999, 998, 997, 996, 995, 994, 500, 100])

        itim = cmor.axis(
            table_entry='time',
            units='months since 2030-1-1',
            length=ntimes,
            interval='1 month')

        ilev = cmor.axis(
            table_entry='plev19',
            units='Pa',
            coord_vals=plevs,
            cell_bounds=None)

        var3d_ids = cmor.variable(
            table_entry='ta',
            units='K',
            axis_ids=numpy.array((ilev, ilon, ilat, itim)),
            missing_value=numpy.array([1.0e28, ], dtype=numpy.float32)[0],
            original_name='cloud')

        cmor.set_quantize(var3d_ids, quantize_mode, quantize_nsd)

        use_deflate = 1 if deflate > 0 else 0
        use_shuffle = 1 if shuffle else 0
        cmor.set_deflate(var3d_ids, use_shuffle, use_deflate,
                         deflate_level=deflate_level)
        cmor.set_zstandard(var3d_ids, zstd_level)

        for it in range(ntimes):

            time = numpy.array((it))
            bnds_time = numpy.array((it, it + 1))
            data3d = numpy.random.random((len(plevs), nlon, nlat)) * 30. + 265.
            data3d = data3d.astype('f')
            cmor.write(
                var_id=var3d_ids,
                data=data3d,
                ntimes_passed=1,
                time_vals=time,
                time_bnds=bnds_time)

        nc_path = cmor.close(var3d_ids, file_name=True)

        if deflate:
            dst_file = f'deflate_level_{str(deflate_level)}'
        else:
            dst_file = f'zstd_level_{str(zstd_level)}'
        if shuffle:
            dst_file += '_shuffle'
        dst_file += f'qmode_{str(quantize_mode)}_nsd_{str(quantize_nsd)}.nc'

        dst_path = os.path.join(out_dir, dst_file)

        shutil.copy(nc_path, dst_path)

        return dst_path

    def testZstandardCompression(self):

        seed = 123
        ntimes = 100

        with tempfile.TemporaryDirectory() as tmp_dir:
            no_compression = self.gen_file(seed, ntimes, 0, True, 0, False, 0, 0, tmp_dir)
            zstd_shuffle = self.gen_file(seed, ntimes, 3, False, 0, True, 0, 0, tmp_dir)

            no_comp_size = os.path.getsize(no_compression)
            zstd_shuffle_size = os.path.getsize(zstd_shuffle)
            print(f'File size without compression: {no_comp_size} bytes')
            print(f'File size with zstandard compression and shuffle: {zstd_shuffle_size} bytes')
            self.assertTrue(zstd_shuffle_size < no_comp_size)

            no_comp_nc = Dataset(no_compression, "r", format="NETCDF4")
            zstd_shuffle_nc = Dataset(zstd_shuffle, "r", format="NETCDF4")

            no_comp_ta = no_comp_nc.variables['ta'][:]
            zstd_shuffle_ta = zstd_shuffle_nc.variables['ta'][:]
            self.assertIsNone(numpy.testing.assert_array_equal(no_comp_ta, zstd_shuffle_ta))

    def testQuantize(self):

        seed = 123
        ntimes = 100

        with tempfile.TemporaryDirectory() as tmp_dir:
            default = self.gen_file(seed, ntimes, 0, True, 1, False, 0, 0, tmp_dir)
            quantized = self.gen_file(seed, ntimes,  0, True, 1, False, 1, 4, tmp_dir)

            default_size = os.path.getsize(default)
            quantized_size = os.path.getsize(quantized)
            print(f'File size without quantization: {default_size} bytes')
            print(f'File size with quantization: {quantized_size} bytes')
            self.assertTrue(quantized_size < default_size)

            default_nc = Dataset(default, "r", format="NETCDF4")
            quantized_nc = Dataset(quantized, "r", format="NETCDF4")

            default_ta = default_nc.variables['ta'][:]
            quantized_ta = quantized_nc.variables['ta'][:]
            self.assertIsNone(numpy.testing.assert_allclose(default_ta, quantized_ta, rtol=1e-4))


if __name__ == '__main__':
    run()



#!/usr/bin/env python
from __future__ import print_function
import argparse
import sys
import glob
import time
import os

if "VERSION" in os.environ.keys():
    last_stable=os.environ['VERSION']
else:
    last_stable = "8.0"

l = time.localtime()
today = "%s.%.2i.%.2i.%.2i.%.2i.%.2i.{{ GIT_DESCRIBE_HASH }}" % (last_stable, l.tm_year, l.tm_mon, l.tm_mday, l.tm_hour, l.tm_min)

parser = argparse.ArgumentParser(
    description='Cleanup your anaconda server',
    formatter_class=argparse.ArgumentDefaultsHelpFormatter)

parser.add_argument("-b", "--branch", default="master",
                    help="branch to use on uvcdat repo")
parser.add_argument(
    "-B",
    "--build",
    default="0",
    help="name of this build")

parser.add_argument("-v", "--version", default=today,
                    help="which version are we building")

parser.add_argument("-l", "--last_stable", default=last_stable,
                    help="which version are we building")

parser.add_argument("-f", "--features", nargs="*",
                    help="features to be enabled", default=[])


args = parser.parse_args(sys.argv[1:])

today2 = "%s.%.2i.%.2i.%.2i.%.2i.%.2i.{{ GIT_DESCRIBE_HASH }}" % (args.last_stable, l.tm_year, l.tm_mon, l.tm_mday, l.tm_hour, l.tm_min)
featured_packages = {}
files = glob.glob("*/meta.yaml.in")
for fnm in files:
    print(fnm, args.version,today)
    with open(fnm) as f:
        s = f.read()
    s = s.replace("@UVCDAT_BRANCH@", args.branch)
    s = s.replace("@BUILD_NUMBER@", args.build)
    if args.version != today:
        s = s.replace("@VERSION@", args.version)
    else:
        s = s.replace("@VERSION@", today2)

    out = []
    for l in s.split("\n"):
        if any([l.find("{{{ %s }}}" % f)>-1 for f in args.features]):
            for f in args.features:
                l = l.replace("{{{ %s }}}" % f,"")
            out.append(l)
        elif l.find("{{{")==-1:  # it's not a unused feature line
            out.append(l)
    with open(fnm[:-3],"w") as f:
        f.write("\n".join(out))



import cmip6_cv._cmip6_cv
atts = """
CMOR_MAX_STRING
CMOR_MAX_ELEMENTS
CMOR_MAX_AXES
CMOR_MAX_VARIABLES
CMOR_MAX_GRIDS
CMOR_MAX_DIMENSIONS
CMOR_MAX_ATTRIBUTES
CMOR_MAX_ERRORS
CMOR_MAX_TABLES
CMOR_MAX_GRID_ATTRIBUTES
CMOR_QUIET
CMOR_EXIT_ON_MAJOR
CMOR_EXIT
CMOR_EXIT_ON_WARNING
CMOR_VERSION_MAJOR
CMOR_VERSION_MINOR
CMOR_VERSION_PATCH
CMOR_CF_VERSION_MAJOR
CMOR_CF_VERSION_MINOR
CMOR_WARNING
CMOR_NORMAL
CMOR_CRITICAL
TABLE_CONTROL_FILENAME
GLOBAL_CV_FILENAME
CMOR_DEFAULT_PATH_TEMPLATE
CMOR_DEFAULT_FILE_TEMPLATE
CMOR_DEFAULT_FURTHERURL_TEMPLATE
FILE_PATH_TEMPLATE
FILE_NAME_TEMPLATE
GLOBAL_ATT_FURTHERINFOURLTMPL
GLOBAL_ATT_MEMBER_ID
CMOR_AXIS_ENTRY_FILE
CMOR_FORMULA_VAR_FILE
"""

for att in atts.split():
    attnm = att
    exec("%s = cmip6_cv._cmip6_cv.getCMOR_defaults_include('%s')" % (att, att))



import cmor
import numpy
import os
import sys

def testUnicode():

    # -------------------------------------------
    # Run CMOR using unicode strings
    # -------------------------------------------

    cmor.setup(inpath=u'Tables', netcdf_file_action=cmor.CMOR_REPLACE)
    cmor.dataset_json(u"Test/CMOR_input_example.json")
    cmor.load_table(u"CMIP6_CFsubhr.json")

    axes = [{u'table_entry': u'time1',
            u'units': u'days since 2000-01-01 00:00:00',
            },
            {u'table_entry': u'site',
            u'units': u'',
            u'coord_vals': [0]},
            {u'table_entry': u'hybrid_height',
            u'units': u'm',
            u'coord_vals': numpy.array(range(2), dtype=numpy.float64),
            u'cell_bounds': [[x, x + 1] for x in range(2)],
            },
            ]

    values = numpy.array([0.5, 0.5], numpy.float32)

    axis_ids = list()
    for axis in axes:
        axis_id = cmor.axis(**axis)
        axis_ids.append(axis_id)

    igrid = cmor.grid([axis_ids[1]], [0.], [0.])
    cmor.zfactor(axis_ids[2], u'b', axis_ids=[axis_ids[2]],
                zfactor_values=numpy.array(range(2), dtype=numpy.float64), 
                zfactor_bounds=[[x - 0.5, x + 0.5] for x in range(2)])

    cmor.zfactor(axis_ids[2], u'orog', u'm', axis_ids=[igrid],
                zfactor_values=[0.] )

    ids_for_var = [axis_ids[0], igrid, axis_ids[2]]
    varid = cmor.variable(u'tnhus',
                        u's-1',
                        ids_for_var,
                        history=u'variable history',
                        missing_value=-99,
                        )

    for time in [x * 1800. / 86400 for x in range(48)]:
        cmor.write(varid, values, time_vals=[time])

    cmor.close()


if __name__ == '__main__':
    testUnicode()



#!/usr/bin/env python
import cmor
import numpy


def main():

    cmor.setup(inpath='Tables',
               netcdf_file_action=cmor.CMOR_REPLACE_3)
    cmor.dataset('pre-industrial control', 'ukmo', 'HadCM3', '360_day',
                 institute_id='ukmo',
                 model_id='HadCM3',
                 history='some global history',
                 forcing='N/A',
                 parent_experiment_id='N/A',
                 branch_time=0.,
                 contact='bob')

    table = 'CMIP5_3hr'
    cmor.load_table(table)
    axes = [{'table_entry': 'time1',
             'units': 'hours since 2000-01-01 00:00:00',
             },
            {'table_entry': 'latitude',
             'units': 'degrees_north',
             'coord_vals': [0],
             'cell_bounds': [-1, 1]},
            {'table_entry': 'longitude',
             'units': 'degrees_east',
             'coord_vals': [90],
             'cell_bounds': [89, 91]},
            ]

    values = numpy.array([1.], numpy.float32) + 200
    axis_ids = list()
    for axis in axes:
        axis_id = cmor.axis(**axis)
        axis_ids.append(axis_id)

    varid = cmor.variable('tas',
                          'K',
                          axis_ids,
                          history='variable history',
                          missing_value=-99,
                          )
    cmor.write(varid, values, time_vals=[0.], time_bnds=[[0, 3.]])

    cmor.close()


if __name__ == '__main__':

    main()



from __future__ import print_function
import cmor
import glob
import unittest



def run():
    unittest.main()

class TestLoadTables(unittest.TestCase):
    def testLoadTables(self):
        tables = glob.glob("Tables/CMIP6*json")
        for table in tables:
            if "formula_terms" in table:
                continue
            cmor.setup(inpath='Tables', netcdf_file_action=cmor.CMOR_REPLACE)
            cmor.dataset_json("Test/CMOR_input_example.json")
            print("Loading table:", table)
            ierr = cmor.load_table(table)
            self.assertEqual(ierr, 0)
            cmor.close()

if __name__ == '__main__':
    run()


import cmor
import numpy

ipth = "Tables"
cmor.setup(inpath=ipth,
           set_verbosity=cmor.CMOR_NORMAL,
           netcdf_file_action=cmor.CMOR_REPLACE,
           logfile=None)

cmor.dataset_json("Test/CMOR_input_example.json")

cmor.load_table("CMIP6_Amon.json")

nlat = 90
dlat = 180 / nlat
nlon = 180
dlon = 360. / nlon

lats = numpy.arange(-90 + dlat / 2., 90, dlat)
blats = numpy.arange(-90, 90 + dlat, dlat)
lons = numpy.arange(0 + dlon / 2., 360., dlon)
blons = numpy.arange(0, 360. + dlon, dlon)

ntime = 12

data = numpy.random.random((ntime, nlat, nlon)) + 280.

itim = cmor.axis(
    table_entry='time', coord_vals=numpy.arange(
        0, ntime, 1), units='month since 2008', cell_bounds=numpy.arange(
            0, ntime + 1, 1))
ilat = cmor.axis(
    table_entry='latitude',
    coord_vals=lats,
    units='degrees_north',
    cell_bounds=blats)
ilon = cmor.axis(
    table_entry='longitude',
    coord_vals=lons,
    units='degrees_east',
    cell_bounds=blons)

iv = cmor.variable(
    table_entry='tas', axis_ids=numpy.array(
        (itim, ilat, ilon)), units='K')

cmor.write(iv, data)



from __future__ import print_function
import sys
import os
try:
    import cdms2
except BaseException:
    print('This test requires cdms2 for I/O')
    sys.exit()

import cmor
import numpy

f = cdms2.open(os.path.join('data/clt.nc'))


pth = os.path.split(os.path.realpath(os.curdir))
if pth[-1] == 'Test':
    ipth = opth = '.'
else:
    ipth = opth = 'Test'
cmor.setup(inpath=ipth,
           set_verbosity=cmor.CMOR_NORMAL,
           netcdf_file_action=cmor.CMOR_REPLACE)

cmor.dataset_json("Test/CMOR_input_example.json")

cmor.load_table("Tables/CMIP6_Amon.json")

s = f("clt", slice(14))
Saxes = s.getAxisList()

axes = []
for ax in Saxes[1:]:
    tmp = cmor.axis(
        ax.id,
        coord_vals=ax[:],
        cell_bounds=ax.getBounds(),
        units=ax.units)
    axes.append(tmp)

# Now creates a dummy HUGE axis for resizing s as really big
factor = 100
nt = s.shape[0] * factor
print('nt is:', nt)
t = numpy.arange(nt)

tmp = cmor.axis(
    'time',
    coord_vals=t,
    units=Saxes[0].units,
    cell_bounds=numpy.arange(
        nt + 1))
axes.insert(0, tmp)
print(axes)
var_id1 = cmor.variable(s.id, s.units, axes)
# the one with 2 at the end is compressed
var_id2 = cmor.variable(s.id, s.units, axes)
sh = list(s.shape)
sh[0] = nt
s = numpy.resize(s, sh)
# s=numpy.where(numpy.greater(s,100.),100,s)
s = numpy.random.random(s.shape) * 10000.
print(s.shape)
cmor.write(var_id1, s)
cmor.close(var_id1)
cmor.write(var_id2, s)

cmor.close()



from __future__ import print_function
import cmor
import numpy


nlat = 360
dlat = 180. / nlat
nlon = 720
dlon = 360. / nlon
nlev = 19
ntimes = 12

lats = numpy.arange(-90 + dlat / 2., 90, dlat)
blats = numpy.arange(-90, 90 + dlat, dlat)
lons = numpy.arange(0 + dlon / 2., 360., dlon)
blons = numpy.arange(0, 360. + dlon, dlon)


cmor.setup(inpath='Tables', netcdf_file_action=cmor.CMOR_REPLACE)
cmor.dataset_json("Test/CMOR_input_example.json")
table = 'CMIP6_Amon.json'
cmor.load_table(table)


ilat = cmor.axis(
    table_entry='latitude',
    coord_vals=lats,
    cell_bounds=blats,
    units='degrees_north')
ilon = cmor.axis(
    table_entry='longitude',
    coord_vals=lons,
    cell_bounds=blons,
    units='degrees_east')
# ,coord_vals=numpy.arange(ntimes,dtype=float),cell_bounds=numpy.arange(ntimes+1,dtype=float),units='months since 2000')
itim = cmor.axis(table_entry='time', units='months since 2010')
ilev = cmor.axis(table_entry='plev19',
                 coord_vals=numpy.array([1000.,
                                         925,
                                         850,
                                         700,
                                         600,
                                         500,
                                         400,
                                         300,
                                         250,
                                         200,
                                         150,
                                         100,
                                         70,
                                         50,
                                         30,
                                         20,
                                         10,
                                         5,
                                         1]),
                 units='hPa')

axes = [itim, ilev, ilat, ilon]

var = cmor.variable(table_entry='ta', units='K', axis_ids=axes)
ntimes = 250

data = numpy.random.random((nlev, nlat, nlon)) * 30 + 273.15

for i in range(ntimes):
    if i % 10 == 0:
        print('Writing time:', i)
    cmor.write(var, data, time_vals=numpy.array(
        [float(i), ]), time_bnds=numpy.array([i, i + 1.]))

print(cmor.close(var_id=var, file_name=True))
cmor.close()


print('hello')



from __future__ import print_function
import cmor
import numpy


def cmor_initialisation():
    cmor.setup(inpath='Tables',
               netcdf_file_action=cmor.CMOR_REPLACE_3,
               create_subdirectories=0)
    cmor.dataset('pre-industrial control', 'ukmo', 'HadCM3', '360_day',
                 institute_id='ukmo',
                 model_id='HadCM3',
                 history='some global history',
                 forcing='N/A',
                 parent_experiment_id='N/A',
                 parent_experiment_rip='N/A',
                 branch_time=0.,
                 contact='bob',
                 outpath='Test')


def setup_data():
    ntimes = 7200
    tvals = numpy.arange(ntimes) * 6.
    tbnds = list(tvals)
    tbnds.append(43200)
    tbnds = numpy.array(tbnds) - 3.
    print("tvals:", tvals)
    print("tbnds:", tbnds)
    import cdtime
    tunits = 'hours since 209-01-01 06:00:00'
    t1 = cdtime.reltime(tvals[0], tunits)
    t2 = cdtime.reltime(tvals[-1], tunits)
    t3 = cdtime.reltime(tbnds[-1], tunits)
    print(t1.tocomp(), t1.tocomp(cdtime.Calendar360))
    print(t2.tocomp(), t2.tocomp(cdtime.Calendar360))
    print(t3.tocomp(), t3.tocomp(cdtime.Calendar360))

    axes = [{'table_entry': 'time1',
             'units': tunits,
             'coord_vals': tvals,
             'cell_bounds': tbnds,
             },
            {'table_entry': 'latitude',
             'units': 'degrees_north',
             'coord_vals': [0],
             'cell_bounds': [-5, 5]},
            {'table_entry': 'longitude',
             'units': 'degrees_east',
             'coord_vals': [0],
             'cell_bounds':[-10, 10]},
            # {'table_entry': 'depth',
            # 'units': 'm',
            # 'coord_vals': [10],
            # 'cell_bounds': [5,15]},
            ]

    values = numpy.ones(ntimes) * 1013.
    return values.astype("f"), axes


def cmor_define_and_write(values, axes):
    table = '/git/cmip5-cmor-tables/Tables/CMIP5_6hrPlev'
    cmor.load_table(table)
    ## lev_axis_id = cmor.axis(**axes[2])
    lon_axis_id = cmor.axis(**axes[2])

    lat_axis_id = cmor.axis(**axes[1])

    time_axis_id = cmor.axis(**axes[0])

    #gid = cmor.grid([site_axis_id,],latitude=numpy.array([-20,]),longitude=numpy.array([150,]))

    axis_ids = [time_axis_id, lat_axis_id, lon_axis_id]
    varid = cmor.variable('psl',
                          'hPa',
                          axis_ids,
                          history='variable history',
                          missing_value=-99,
                          )

    cmor.write(varid, values)
    return varid


def main():

    cmor_initialisation()
    values, axes = setup_data()
    vid = cmor_define_and_write(values, axes)
    print(cmor.close(var_id=vid, file_name=True))


if __name__ == '__main__':

    main()



import numpy
import os
import warnings
import sys
import six

from cmor import cmor_const
from cmor import _cmor
from cmor._cmor import CMORError

global climatology 
climatology = False

try:
    import cdtime
    has_cdtime = True
except BaseException:
    has_cdtime = False

try:
    import cdms2
    has_cdms2 = True
except BaseException:
    has_cdms2 = False

try:
    import MV2
    has_MV2 = True
except BaseException:
    has_MV2 = False

try:
    import numpy.oldnumeric.ma.MaskedArray
    has_oldma = True
except BaseException:
    has_oldma = False

def get_terminate_signal():
    """ Return the integer code currently used by CMOR for system signal when issuing an error
    -999 means not set yet, once cmor_setup is called -999 is turned to SIGTERM"""
    return _cmor.get_terminate_signal()

def set_terminate_signal(signal):
    """Sets the signal code to be used by CMOR when issuing an error
    int has to match whateve rint the C uses"""
    _cmor.set_terminate_signal(signal)


def time_varying_grid_coordinate(
        grid_id, table_entry, units, data_type='f', missing_value=None):
    """ Create a cmor variable for grid coordinates in case of time varying grids
    Usage:
    coord_grid_id = grid_time_varying_coordinate(
    grid_id, table_entry,units,data_type='f',missing_value=None)
    Where:
    grid_id : The grid_id return by a call to cmor.grid
    table_entry: The name of the variable in the CMOR table
    units: variable units
    data_type: data type of the missing_value, which must be the same as the type of the array  that will be passed to cmor_write.
    The options are: 'd' (double), 'f' (float), 'l' (long) or 'i' (int).
    missing_value : scalar that is used to indicate missing data for this variable.
    It must be the same type as the data that will be passed to cmor_write.
    This missing_value will in general be replaced by a standard missing_value specified in the MIP table.
    If there are no missing data, and the user chooses not to declare the missing value, then this argument may be either
    omitted or assigned the value 'none' (i.e., missing_value='none').
    """
    if not isinstance(table_entry, six.string_types):
        raise Exception(
            "Error you must pass a string for the variable table_entry")

    if not isinstance(units, six.string_types):
        raise Exception("Error you must pass a string for the variable units")
    if not isinstance(data_type, six.string_types):
        raise Exception("error data_type must be a string")
    data_type = data_type.lower()
    if data_type == 's' or data_type == 'u':
        data_type = 'c'
    if data_type not in ["c", "d", "f", "l", "i"]:
        raise Exception(
            'error unknown data_type: "%s", must be one of: "c","d","f","l","i"')

    if not isinstance(grid_id, (int, numpy.int32, numpy.int64)):
        raise Exception("error grid_id must be an integer")

    grid_id = int(grid_id)

    if missing_value is not None:
        if not isinstance(missing_value, (float, int, numpy.float32, numpy.float64, 
                                          numpy.int32, numpy.int64)):
            raise Exception(
                "error missing_value must be a number, you passed: %s" %
                type(missing_value))
        missing_value = float(missing_value)

    return _cmor.time_varying_grid_coordinate(
        grid_id, table_entry, units, str.encode(data_type), missing_value)


def _to_numpy(vals, message):
    if isinstance(vals, (list, tuple)):
        vals = numpy.ascontiguousarray(vals)
    elif not isinstance(vals, numpy.ndarray):
        try:
            vals = numpy.ascontiguousarray(vals.filled())
        except BaseException:
            raise Exception(
                "Error could not convert %s to a numpy array" %
                message)

    return vals


def grid(axis_ids, latitude=None, longitude=None,
         latitude_vertices=None, longitude_vertices=None, nvertices=None):
    """ Creates a cmor grid
    Usage:
    grid_id = grid(
    axis_ids,
    latitude,
    longitude,
    latitude_vertices=None,
    longitude_vertices=None)
    Where:
    axis_ids : array contianing the axes ids for this grid.
    latitude/longitude: the values for longitude/latitude arrays (unless it is a time varying grid)
    latitude_vertices/longitude_vertices: coordinates of vertices for each latitude/latitude (unless it is a time varying grid)
    """
    if numpy.ma.isMA(axis_ids):
        axis_ids = numpy.ascontiguousarray(axis_ids.filled())
    elif has_oldma and numpy.oldnumeric.ma.isMA(axis_ids):
        axis_ids = numpy.ascontiguousarray(axis_ids.filled())
    elif has_cdms2 and cdms2.isVariable(axis_ids):
        axis_ids = numpy.ascontiguousarray(axis_ids.filled())
    elif has_cdms2 and cdms2.isVariable(axis_ids):
        axis_ids = numpy.ascontiguousarray(axis_ids.filled())
    elif isinstance(axis_ids, (list, tuple)):
        axis_ids = numpy.ascontiguousarray(axis_ids)
    elif not isinstance(axis_ids, numpy.ndarray):
        raise Exception(
            "Error could not convert axis_ids list to a numpy array")

    if numpy.ndim(axis_ids) > 1:
        raise Exception("error axes list/array must be 1D")

    if latitude is not None:
        latitude = _to_numpy(latitude, 'latitude')

        if numpy.ndim(latitude) != len(axis_ids):
            raise Exception(
                "latitude's rank does not match number of axes passed via axis_ids")

        data_type = latitude.dtype.char
        nvert = 0
        if not data_type in ['d', 'f', 'i', 'l']:
            raise Exception(
                "latitude array must be of data_type int32, int64, float32, or float64")

        longitude = _to_numpy(longitude, 'longitude')

        if numpy.ndim(longitude) != len(axis_ids):
            raise Exception(
                "longitude's rank does not match number of axes passed via axis_ids")

    # print 'longitude data_type:',longitude.dtype.char
        if longitude.dtype.char != data_type:
            longitude = longitude.astype(data_type)
    elif longitude is not None:
        raise Exception("latitude and longitude must be BOTH an array or None")
    else:
        data_type = 'f'
        if nvertices is None:
            nvert = 0
        else:
            nvert = nvertices

    if latitude_vertices is not None:
        latitude_vertices = _to_numpy(latitude_vertices, 'latitude_vertices')

        if numpy.ndim(latitude_vertices) != len(axis_ids) + 1:
            raise Exception(
                "latitude_vertices's rank does not match number of axes passed via axis_ids +1 (for vertices)")
# print 'latitude_vert data_type:',latitude_vertices.dtype.char
        if latitude_vertices.dtype.char != data_type:
            latitude_vertices = latitude_vertices.astype(data_type)
        nvert = latitude_vertices.shape[-1]
        if nvertices is not None:
            if nvert != nvertices:
                raise Exception(
                    "you passed nvertices as: %i, but from your latitude_vertices it seems to be: %i" %
                    (nvertices, nvert))

    if longitude_vertices is not None:
        longitude_vertices = _to_numpy(
            longitude_vertices, 'longitude_vertices')
        if numpy.ndim(longitude_vertices) != len(axis_ids) + 1:
            raise Exception(
                "longitude_vertices's rank does not match number of axes passed via axis_ids +1 (for vertices)")
# print 'longitude_vert data_type:',longitude_vertices.dtype.char
        if longitude_vertices.dtype.char != data_type:
            longitude_vertices = longitude_vertices.astype(data_type)
        nvert2 = longitude_vertices.shape[-1]
        if latitude_vertices is None:
            nvert = nvert2
        elif nvert != nvert2:
            raise Exception(
                "error in shape longitude_vertices and latitude_vertices seem to have different # of vertices: %i vs %i, %s" %
                (nvert, nvert2, str(
                    longitude_vertices.shape)))
        if nvertices is not None:
            if nvert != nvertices:
                raise Exception(
                    "you passed nvertices as: %i, but from your longitude_vertices it seems to be: %i" %
                    (nvertices, nvert))


# if area is not None:
# if not isinstance(area,numpy.ndarray):
# try:
##                 area = numpy.ascontiguousarray(area.filled())
# except:
##                 raise Exception, "Error could not convert area to a numpy array"
# if numpy.rank(area)!=len(axis_ids):
##                 raise Exception, "area's rank does not match number of axes passed via axis_ids"
# if area.dtype.char!=data_type:
##             area = area.astype(data_type)
    n = len(axis_ids)
    axis_ids = axis_ids.astype('i')
    return _cmor.grid(n, axis_ids, str.encode(data_type), latitude, longitude,
                      nvert, latitude_vertices, longitude_vertices)


def set_grid_mapping(grid_id, mapping_name, parameter_names,
                     parameter_values=None, parameter_units=None):
    """Sets the grid mapping for CF convention
    Usage:
       set_grid_mapping(grid_id,mapping_name,parameter_names,parameter_values,parameter_units)
    Where:
       grid_id :: grid_id return by cmor.grid
       mapping_name     :: name of the mapping (see CF conventions)
       parameter_names  :: list of parameter names or dictionary with name as keys and values can be either [value,units] list/tuple or dictionary with "values"/"units" as keys
       parameter_values :: array/list of parameter values in the same order of parameter_names (ignored if parameter_names is ditcionary)
       parameter_units  :: array/list of parameter units  in the same order of parameter_names (ignored if parameter_names is ditcionary)
    """
    if sys.version_info < (3, 0):
        if not isinstance(grid_id, (numpy.int32, numpy.int64, int, long)):
            raise Exception("grid_id must be an integer: %s" % type(grid_id))
    else:
        if not isinstance(grid_id, (numpy.int32, numpy.int64, int)):
            raise Exception("grid_id must be an integer: %s" % type(grid_id))
        grid_id = int(grid_id)
    if not isinstance(mapping_name, six.string_types):
        raise Exception("mapping name must be a string")

    if isinstance(parameter_names, dict):
        pnms = []
        pvals = []
        punit = []
        for k in list(parameter_names.keys()):
            pnms.append(k)
            val = parameter_names[k]
            if isinstance(val, dict):
                ks = list(val.keys())
                if not 'value' in ks or not 'units' in ks:
                    raise Exception(
                        "error parameter_names key '%s' dictionary does not contain both 'units' and 'value' keys" %
                        k)
                pvals.append(val['value'])
                punit.append(val['units'])
            elif isinstance(val, (list, tuple)):
                if len(val) > 2:
                    raise Exception(
                        "parameter_names '%s' as more than 2 values" %
                        k)
                for v in val:
                    if isinstance(v, six.string_types):
                        punit.append(v)
                    try:
                        pvals.append(float(v))
                    except BaseException:
                        pass
                if len(pvals) != len(punit) or len(pvals) != len(pnms):
                    raise Exception(
                        "could not figure out values for parameter_name: '%s' " %
                        k)
            else:
                raise Exception(
                    "could not figure out values for parameter_name: '%s' " %
                    k)
    elif isinstance(parameter_names, (list, tuple)):
        pnms = list(parameter_names)
        # now do code for parameter_units
        if parameter_values is None:
            raise Exception(
                "you must pass a list or array for parameter_values")
        if parameter_units is None:
            raise Exception("you must pass a list for parameter_units")
        if not isinstance(parameter_units, (list, tuple)):
            raise Exception("you must pass a list for parameter_units")
        if len(parameter_units) != len(pnms):
            raise Exception(
                "length of parameter_units list does not match length of parameter_names")
        punit = list(parameter_units)
        if isinstance(parameter_values, (list, tuple)):
            pvals = list(parameter_values)
        else:
            try:
                pvals = numpy.ascontiguousarray(parameter_values.filled())
            except BaseException:
                raise Exception(
                    "Error could not convert parameter_values to a numpy array")
        if len(pvals) != len(parameter_names):
            raise Exception(
                "length of parameter_values list does not match length of parameter_names")
    else:
        raise Exception("parameter_names must be either dictionary or list")

    pvals = numpy.ascontiguousarray(pvals).astype('d')
    return _cmor.set_grid_mapping(grid_id, mapping_name, pnms, pvals, punit)

def set_climatology(entry):
    global climatology 
    if( entry == True ):
      climatology = True 
    else:
      climatology = False

def get_climatology():
    global climatology 
    return climatology

def axis(table_entry, units=None, length=None,
         coord_vals=None, cell_bounds=None, interval=None):
    """ Creates an cmor_axis
    Usage:
    axis_id = axis(
    table_entry,
    units=None,
    length=None,
    coord_vals=None,
    cell_bounds=None,
    interval=None)
    Where:
    table_entry: table_entry in the cmor table
    units: the axis units
    length: the number of coord_vals to actuall y use, or simply the number of coord_vals in case in index_only axes
    coord_vals: cmds2 axis or numpy/MV2 array (1D)
    cell_bounds: numpy or MV2 array, if coord_vals is a cdms2 axis then will try to obtain bounds from it
    interval: a string used for time axes only (???)
    """
    if not isinstance(table_entry, six.string_types):
        raise Exception(
            "You need to pass a table_entry to match in the cmor table")

    if(table_entry in ['time2', 'time3']):
        set_climatology(True)

    if coord_vals is None:
        if cell_bounds is not None:
            raise Exception("you passed cell_bounds but no coords")
    else:
        if has_cdms2 and isinstance(coord_vals, cdms2.axis.TransientAxis):
            if units is None:
                if hasattr(coord_vals, "units"):
                    units = coord_vals.units
            if cell_bounds is None:
                cell_bounds = coord_vals.getBounds()

            if interval is None and hasattr(coord_vals, "interval"):
                interval = coord_vals.interval
            coord_vals = numpy.ascontiguousarray(coord_vals[:])
        elif isinstance(coord_vals, (list, tuple)):
            coord_vals = numpy.ascontiguousarray(coord_vals)
        elif has_cdms2 and cdms2.isVariable(coord_vals):
            if units is None:
                if hasattr(coord_vals, "units"):
                    units = coord_vals.units
            if interval is None and hasattr(coord_vals, "interval"):
                interval = coord_vals.interval
            coord_vals = numpy.ascontiguousarray(coord_vals.filled())
        elif has_oldma and numpy.oldnumeric.ma.isMA(coord_vals):
            coord_vals = numpy.ascontiguousarray(coord_vals.filled())
        elif numpy.ma.isMA(coord_vals):
            coord_vals = numpy.ascontiguousarray(coord_vals.filled())

        if not isinstance(coord_vals, numpy.ndarray):
            raise Exception(
                "Error coord_vals must be an array or cdms2 axis or list/tuple")

        if numpy.ndim(coord_vals) > 1:
            raise Exception("Error, you must pass a 1D array!")

    if numpy.ma.isMA(cell_bounds):
        cell_bounds = numpy.ascontiguousarray(cell_bounds.filled())
    elif has_oldma and numpy.oldnumeric.ma.isMA(cell_bounds):
        cell_bounds = numpy.ascontiguousarray(cell_bounds.filled())
    elif has_cdms2 and cdms2.isVariable(cell_bounds):
        cell_bounds = numpy.ascontiguousarray(cell_bounds.filled())
    elif has_cdms2 and cdms2.isVariable(cell_bounds):
        cell_bounds = numpy.ascontiguousarray(cell_bounds.filled())
    elif isinstance(cell_bounds, (list, tuple)):
        cell_bounds = numpy.ascontiguousarray(cell_bounds)

    if cell_bounds is not None:
        if numpy.ndim(cell_bounds) > 2:
            raise Exception("Error cell_bounds rank must be at most 2")
        if numpy.ndim(cell_bounds) == 2:
            if cell_bounds.shape[0] != coord_vals.shape[0]:
                raise Exception(
                    "Error, coord_vals and cell_bounds do not have the same length")
            if cell_bounds.shape[1] != 2:
                raise Exception(
                    "Error, cell_bounds' second dimension must be of length 2")
            cbnds = 2
            cell_bounds = numpy.ascontiguousarray(numpy.ravel(cell_bounds))
        else:
            cbnds = 1
            if len(cell_bounds) != len(coord_vals) + 1:
                raise Exception(
                    "error cell_bounds are %i long and axes coord_vals are %i long this is not consistent" %
                    (len(cell_bounds), len(coord_vals)))
    else:
        cbnds = 0

    if coord_vals is not None:
        l = len(coord_vals)
        data_type = coord_vals.dtype.char[0]

        if not data_type in ['i', 'l', 'f', 'd', 'S', 'U']:
            raise Exception("error allowed data data_type are: int32, int64, float32, float64, string")

        if data_type == 'S' or data_type == 'U':
            if data_type == 'S':
                coord_vals = numpy.char.decode(coord_vals, encoding='utf-8')
            data_type = 'c'
            cbnds = 0
    else:
        l = 0
        data_type = 'd'

    if cell_bounds is not None:
        if data_type != cell_bounds.dtype.char:
            cell_bounds = cell_bounds.astype(data_type)

    if units is None:
        if coord_vals is not None:
            raise Exception(
                "Error you need to provide the units your coord_vals are in")
        else:
            units = "1"

    if interval is None:
        interval = ""

    if length is not None:
        l = int(length)

    return _cmor.axis(table_entry, units, l, coord_vals,
                      str.encode(data_type), cell_bounds, cbnds, interval)


def variable(table_entry, units, axis_ids, data_type='f', missing_value=None,
             tolerance=1.e-4, positive=None, original_name=None, history=None, comment=None):

    if not isinstance(table_entry, six.string_types):
        raise Exception(
            "Error you must pass a string for the variable table_entry")

    if not isinstance(units, six.string_types):
        raise Exception("Error you must pass a string for the variable units")

    if original_name is not None:
        if not isinstance(original_name, six.string_types):
            raise Exception(
                "Error you must pass a string for the variable original_name")
    else:
        original_name = ""

    if history is not None:
        if not isinstance(history, six.string_types):
            raise Exception(
                "Error you must pass a string for the variable history")
    else:
        history = ""

    if comment is not None:
        if not isinstance(comment, six.string_types):
            raise Exception(
                "Error you must pass a string for the variable comment")
    else:
        comment = ""

    if numpy.ma.isMA(axis_ids):
        axis_ids = numpy.ascontiguousarray(axis_ids.filled())
    elif has_oldma and numpy.oldnumeric.ma.isMA(axis_ids):
        axis_ids = numpy.ascontiguousarray(axis_ids.filled())
    elif has_cdms2 and cdms2.isVariable(axis_ids):
        axis_ids = numpy.ascontiguousarray(axis_ids.filled())
    elif isinstance(axis_ids, (list, tuple)):
        axis_ids = numpy.ascontiguousarray(axis_ids)
    elif not isinstance(axis_ids, numpy.ndarray):
        raise Exception(
            "Error could not convert axis_ids list to a numpy array")

    if numpy.ndim(axis_ids) > 1:
        raise Exception("error axis_ids list/array must be 1D")

    if not isinstance(data_type, six.string_types):
        raise Exception("error data_type must be a string")
    data_type = data_type.lower()
    if data_type == 's' or data_type == 'u':
        data_type = 'c'
    if not data_type in ["c", "d", "f", "l", "i"]:
        raise Exception(
            'error unknown data_type: "%s", must be one of: "c","d","f","l","i"')

    ndims = len(axis_ids)

    if positive is None:
        positive = ""
    else:
        positive = str(positive)

    if history is None:
        history = ""
    else:
        history = str(history)

    if comment is None:
        comment = ""
    else:
        comment = str(comment)

    if not isinstance(tolerance, (float, int, numpy.float32, numpy.float64, 
                                  numpy.int32, numpy.int64)):
        raise Exception("error tolerance must be a number")

    tolerance = float(tolerance)

    if missing_value is not None:
        if not isinstance(missing_value, (float, int, numpy.float32, numpy.float64, 
                                          numpy.int32, numpy.int64)):
            raise Exception(
                "error missing_value must be a number, you passed: %s" %
                repr(missing_value))

        missing_value = float(missing_value)

    axis_ids = axis_ids.astype('i')
    return _cmor.variable(table_entry, units, ndims, axis_ids, str.encode(data_type),
                          missing_value, tolerance, positive, original_name, history, comment)


def zfactor(zaxis_id, zfactor_name, units="", axis_ids=None,
            data_type=None, zfactor_values=None, zfactor_bounds=None):

    if not isinstance(zaxis_id, (int, numpy.int32, numpy.int64)):
        raise Exception("error zaxis_id must be a number")
    zaxis_id = int(zaxis_id)

    if not isinstance(zfactor_name, six.string_types):
        raise Exception(
            "Error you must pass a string for the variable zfactor_name")

    if not isinstance(units, six.string_types):
        raise Exception("Error you must pass a string for the variable units")

    if numpy.ma.isMA(axis_ids):
        axis_ids = numpy.ascontiguousarray(axis_ids.filled())
    elif has_oldma and numpy.oldnumeric.ma.isMA(axis_ids):
        axis_ids = numpy.ascontiguousarray(axis_ids.filled())
    elif has_cdms2 and cdms2.isVariable(axis_ids):
        axis_ids = numpy.ascontiguousarray(axis_ids.filled())
    elif has_cdms2 and cdms2.isVariable(axis_ids):
        axis_ids = numpy.ascontiguousarray(axis_ids.filled())
    elif isinstance(axis_ids, (list, tuple)):
        axis_ids = numpy.ascontiguousarray(axis_ids)
    elif axis_ids is None:
        pass
    elif isinstance(axis_ids, (int, numpy.int32, numpy.int64)):
        axis_ids = numpy.array([axis_ids, ])
    elif not isinstance(axis_ids, numpy.ndarray):
        raise Exception(
            "Error could not convert axis_ids list to a numpy array")

    if numpy.ndim(axis_ids) > 1:
        raise Exception("error axis_ids list/array must be 1D")

    if axis_ids is None:
        ndims = 0
    else:
        ndims = len(axis_ids)

    if zfactor_values is not None:
        if isinstance(zfactor_values, (float, int, numpy.float32, numpy.int32)):
            zfactor_values = numpy.array((zfactor_values,))
        elif numpy.ma.isMA(zfactor_values):
            zfactor_values = numpy.ascontiguousarray(zfactor_values.filled())
        elif has_oldma and numpy.oldnumeric.ma.isMA(zfactor_values):
            zfactor_values = numpy.ascontiguousarray(zfactor_values.filled())
        elif has_cdms2 and cdms2.isVariable(zfactor_values):
            zfactor_values = numpy.ascontiguousarray(zfactor_values.filled())
        elif isinstance(zfactor_values, (list, tuple)):
            zfactor_values = numpy.ascontiguousarray(zfactor_values)
        elif not isinstance(zfactor_values, numpy.ndarray):
            raise Exception(
                "Error could not convert zfactor_values to a numpy array")

        if data_type is None:
            try:
                data_type = zfactor_values.dtype.char
            except BaseException:
                if isinstance(zfactor_values, (float, numpy.float32)):
                    data_type = 'f'
                elif isinstance(zfactor_values, (int, numpy.int32)):
                    data_type = 'd'
                else:
                    raise Exception(
                        "Error unknown data_type for zfactor_values: %s" %
                        repr(zfactor_values))
    elif data_type is None:
        data_type = 'd'

    if not isinstance(data_type, six.string_types):
        raise Exception("error data_type must be a string")
    data_type = data_type.lower()
    if data_type == 's' or data_type == 'u':
        data_type = 'c'
    if not data_type in ["c", "d", "f", "l", "i"]:
        raise Exception(
            'error unknown data_type: "%s", must be one of: "c","d","f","l","i"')

    if zfactor_bounds is not None:
        if numpy.ma.isMA(zfactor_bounds):
            zfactor_bounds = numpy.ascontiguousarray(zfactor_bounds.filled())
        elif has_oldma and numpy.oldnumeric.ma.isMA(zfactor_bounds):
            zfactor_bounds = numpy.ascontiguousarray(zfactor_bounds.filled())
        elif has_cdms2 and cdms2.isVariable(zfactor_bounds):
            zfactor_bounds = numpy.ascontiguousarray(zfactor_bounds.filled())
        elif isinstance(zfactor_bounds, (list, tuple)):
            zfactor_bounds = numpy.ascontiguousarray(zfactor_bounds)
        elif not isinstance(zfactor_bounds, numpy.ndarray):
            raise Exception(
                "Error could not convert zfactor_bounds to a numpy array")
        if numpy.ndim(zfactor_bounds) > 2:
            raise Exception("error zfactor_bounds must be rank 2 at most")
        elif numpy.ndim(zfactor_bounds) == 2:
            if zfactor_bounds.shape[1] != 2:
                raise Exception(
                    "error zfactor_bounds' 2nd dimension must be of length 2")
            bnds = []
            b = zfactor_bounds[0]
            for i in range(zfactor_bounds.shape[0]):
                b = zfactor_bounds[i]
                bnds.append(b[0])
                if (i < zfactor_bounds.shape[0] -
                        1) and (b[1] != zfactor_bounds[i + 1][0]):
                    raise Exception(
                        "error zfactor_bounds have gaps between them")
            bnds.append(zfactor_bounds[-1][1])
            zfactor_bounds = numpy.array(bnds)
            
    if axis_ids is not None:
        axis_ids = axis_ids.astype('i')

    return _cmor.zfactor(zaxis_id, zfactor_name, units,
                         ndims, axis_ids, str.encode(data_type), zfactor_values, zfactor_bounds)


def write(var_id, data, ntimes_passed=None, file_suffix="",
          time_vals=None, time_bnds=None, store_with=None):
    """ write data to a cmor variable
    Usage:
    ierr = write(var_id,data,ntimes_passed=None,file_suffix="",time_vals=None,time_bnds=None,store_with=None
    """
    if not isinstance(var_id, (int, numpy.int32, numpy.int64)):
        raise Exception("error var_id must be an integer")
    var_id = int(var_id)

    if not isinstance(file_suffix, six.string_types):
        raise Exception("Error file_suffix must be a string")

    if store_with is not None:
        if not isinstance(store_with, (int, numpy.int32, numpy.int64)):
            raise Exception("error store_with must be an integer")
        store_with = int(store_with)

    if numpy.ma.isMA(data):
        data = numpy.ascontiguousarray(data.filled())
    elif has_oldma and numpy.oldnumeric.ma.isMA(data):
        data = numpy.ascontiguousarray(data.filled())
    elif has_cdms2 and cdms2.isVariable(data):
        if time_vals is None:
            time_vals = data.getTime()
        data = numpy.ascontiguousarray(data.filled())
    elif isinstance(data, (list, tuple)):
        data = numpy.ascontiguousarray(data)
    elif not isinstance(data, numpy.ndarray):
        raise Exception("Error could not convert data to a numpy array")

    if time_vals is None:
        pass
    elif numpy.ma.isMA(time_vals):
        time_vals = numpy.ascontiguousarray(time_vals.filled())
    elif has_oldma and numpy.oldnumeric.ma.isMA(time_vals):
        time_vals = numpy.ascontiguousarray(time_vals.filled())
    elif has_cdms2 and isinstance(time_vals, cdms2.axis.TransientAxis):
        if time_bnds is None:
            time_bnds = time_vals.getBounds()
        time_vals = numpy.ascontiguousarray(time_vals[:])
    elif has_cdms2 and cdms2.isVariable(time_vals):
        time_vals = numpy.ascontiguousarray(time_vals.filled())
    elif isinstance(time_vals, (list, tuple)):
        time_vals = numpy.ascontiguousarray(time_vals)
    elif not isinstance(time_vals, numpy.ndarray):
        try:
            time_vals = numpy.ascontiguousarray(time_vals)
        except BaseException:
            raise Exception(
                "Error could not convert time_vals to a numpy array")

    if time_vals is not None:
        data_type = time_vals.dtype.char
        if not data_type in ['f', 'd', 'i', 'l']:
            raise Exception(
                "Error time_vals data_type must one of: int32, int64, float32, float64. Please convert first")
        time_vals = time_vals.astype("d")

    if ntimes_passed is None:
        if time_vals is None:
            ntimes_passed = 0
        else:
            ntimes_passed = len(time_vals)
    if not isinstance(ntimes_passed, (int, numpy.int32, numpy.int64)):
        raise Exception("error ntimes_passed must be an integer")
    ntimes_passed = int(ntimes_passed)

    # At that ponit we check that shapes matches!
    goodshape = _cmor.get_original_shape(var_id, 1)
    osh = data.shape
    ogoodshape = list(goodshape)
    sh = list(osh)
    j = 0
    while sh.count(1) > 0:
        sh.remove(1)
    while goodshape.count(1) > 0:
        goodshape.remove(1)
    while goodshape.count(0) > 0:
        if( len(goodshape) == len(sh)):
            index = goodshape.index(0)
            del sh[index]
            del goodshape[index]
        else:  # assume time==1 was removed
            goodshape.remove(0)

    for i in range(len(goodshape)):
        if sh[j] != goodshape[i]:
            if goodshape[i] != 1:
                msg = "Error: your data shape (%s) does not match the expected variable shape (%s)\nCheck your variable dimensions before caling cmor_write" % (str(osh), str(ogoodshape))
                warnings.warn(msg)
        j += 1

    
    # Check if there is enough data for the number of times passed
    if ntimes_passed < 0:
        raise Exception("ntimes_passed must be a positive integer")

    expected_size = ntimes_passed
    for d in goodshape:
        expected_size *= d
    passed_size = 1
    for d in osh:
        passed_size *= d
    if expected_size > passed_size:
        raise Exception("not enough data is being passed for the number of times passed")

    data = numpy.ascontiguousarray(numpy.ravel(data))

    if time_bnds is not None: 
        if numpy.ma.isMA(time_bnds):
            time_bnds = numpy.ascontiguousarray(time_bnds.filled())
        elif has_oldma and numpy.oldnumeric.ma.isMA(time_bnds):
            time_bnds = numpy.ascontiguousarray(time_bnds.filled())
        elif has_cdms2 and cdms2.isVariable(time_bnds):
            if time_vals is None:
                time_vals = time_bnds.getTime()
            time_bnds = numpy.ascontiguousarray(time_bnds.filled())
        elif isinstance(time_bnds, (list, tuple)):
            time_bnds = numpy.ascontiguousarray(time_bnds)
        elif not isinstance(time_bnds, numpy.ndarray):
            raise Exception(
                "Error could not convert time_bnds to a numpy array")

        if numpy.ndim(time_bnds) > 2:
            raise Exception("bounds rank cannot be greater than 2")
        elif numpy.ndim(time_bnds) == 2:
            if time_bnds.shape[1] != 2:
                raise Exception(
                    "error time_bnds' 2nd dimension must be of length 2")
            bnds = []
            if time_bnds.shape[0] > 1 and get_climatology() is False:
                _check_time_bounds_contiguous(time_bnds)
                bnds = _flatten_time_bounds(time_bnds)
            else:
                bnds = time_bnds.ravel()
            time_bnds = numpy.array(bnds)
        else:  # ok it is a rank 1!
            if numpy.ndim(time_vals) == 0:
                ltv = 1
            else:
                ltv = len(time_vals)
            if len(time_bnds) != ltv + 1:
                raise Exception(
                    "error time_bnds if 1D must be 1 elt greater than time_vals, you have %i vs %i" %
                    (len(time_bnds), ltv))
            bnds = []
            for i in range(ltv):
                bnds.append([time_bnds[i], time_bnds[i + 1]])
            bnds = numpy.array(bnds)
            bnds = _flatten_time_bounds(bnds)
            time_bnds = numpy.array(bnds)

    if time_bnds is not None:
        data_type = time_bnds.dtype.char
        if not data_type in ['f', 'd', 'i', 'l']:
            raise Exception(
                "Error time_bnds data_type must one of: int32, int64, float32, float64. Please convert first")
        time_bnds = time_bnds.astype("d")

    data_type = data.dtype.char
    if not data_type in ['f', 'd', 'i', 'l']:
        raise Exception(
            "Error data data_type must one of: int32, int64, float32, float64. Please convert first")

    return _cmor.write(var_id, data, data_type, file_suffix, ntimes_passed,
                       time_vals, time_bnds, store_with)


def _check_time_bounds_contiguous(time_bnds):
    '''
    checks that time bounds are contiguous
    '''
    for i in range(time_bnds.shape[0] - 1):
        b = time_bnds[i]
        if b[1] != time_bnds[i + 1][0]:
            raise Exception("error time_bnds have gaps between them")


def _flatten_time_bounds(time_bnds):
    '''
    return a 1-d list of the time_bnds flattened appropriate for the C call
    '''
    bnds = list()
    for i in range(time_bnds.shape[0]):
        bnds.extend([time_bnds[i][0], time_bnds[i][1]])
    return bnds


def setup(inpath='.', netcdf_file_action=cmor_const.CMOR_PRESERVE, set_verbosity=cmor_const.CMOR_NORMAL,
          exit_control=cmor_const.CMOR_NORMAL, logfile=None, create_subdirectories=1):
    """
    Usage cmor_setup(inpath='.',netcdf_file_action=cmor.CMOR_PRESERVE,set_verbosity=cmor.CMOR_NORMAL,exit_control=cmor.CMOR_NORMAL)
    Where:
    path:                  Alternate directory where to find tables if not in current directory
    netcdf_file_action:    What to do when opening the netcdf file, valid options are:
                           CMOR_PRESERVE, CMOR_APPEND, CMOR_REPLACE, CMOR_PRESERVE_4, CMOR_APPEND_4, CMOR_REPLACE_4, CMOR_PRESERVE_3, CMOR_APPEND_3 or CMOR_REPLACE_3
                           The _3 means netcdf will be created in the old NetCDF3 format (no compression nor chunking), _4 means use NetCDF4 classic format. No _ is equivalent to _3

    set_verbosity:         CMOR_QUIET or CMOR_NORMAL
    exit_control:          CMOR_EXIT_ON_WARNING, CMOR_EXIT_ON_MAJOR, CMOR_NORMAL
    create_subdirectories: 1 to create subdirectories structure, 0 to dump files to the directory specified by the "outpath" attribute 
                           in the user input JSON file passed to cmor_dataset_json
"""
    if not isinstance(exit_control, int) or not exit_control in [
            cmor_const.CMOR_EXIT_ON_WARNING, cmor_const.CMOR_EXIT_ON_MAJOR, cmor_const.CMOR_NORMAL]:
        raise Exception(
            "exit_control must an integer valid values are: CMOR_EXIT_ON_WARNING, CMOR_EXIT_ON_MAJOR, CMOR_NORMAL")

    if not isinstance(netcdf_file_action, int) or not netcdf_file_action in [cmor_const.CMOR_PRESERVE, cmor_const.CMOR_APPEND, cmor_const.CMOR_REPLACE,
                                                                             cmor_const.CMOR_PRESERVE_3, cmor_const.CMOR_APPEND_3, cmor_const.CMOR_REPLACE_3, cmor_const.CMOR_PRESERVE_4, cmor_const.CMOR_APPEND_4, cmor_const.CMOR_REPLACE_4]:
        raise Exception("netcdf_file_action must be an integer. Valid values are: CMOR_PRESERVE, CMOR_APPEND, CMOR_REPLACE, CMOR_PRESERVE_3, CMOR_APPEND_3 or CMOR_REPLACE_3, CMOR_PRESERVE_4, CMOR_APPEND_4 or CMOR_REPLACE_4")

    if not isinstance(set_verbosity, int) or not set_verbosity in [
            cmor_const.CMOR_QUIET, cmor_const.CMOR_NORMAL]:
        raise Exception(
            "set_verbosity must an integer valid values are: CMOR_QUIET, CMOR_NORMAL")

    if not isinstance(inpath, str) and not os.path.exists(inpath):
        raise Exception("path must be a Valid path")
    if logfile is None:
        logfile = ""

    if not create_subdirectories in [0, 1]:
        raise Exception("create_subdirectories must be 0 or 1")
    return _cmor.setup(inpath, netcdf_file_action, set_verbosity,
                       exit_control, logfile, create_subdirectories)


def load_table(table):
    """ loads a cmor table
    Usage:
    load_table(table)
    """
    if not isinstance(table, six.string_types):
        raise Exception("Error, must pass a string")
# if not os.path.exists(table):
##         raise Exception, "Error, the table you specified (%s) does not exists" % table
    return _cmor.load_table(table)


def dataset_json(rcfile):
    """ load dataset JSON file
    Usage:
    dataset_json(rcfile)
    """
    if not isinstance(rcfile, six.string_types):
        raise Exception("Error, must pass a string")
# if not os.path.exists(table):
##         raise Exception, "Error, the table you specified (%s) does not exists" % table
    return _cmor.dataset_json(rcfile)


def set_table(table):
    if not isinstance(table, int):
        raise Exception("error you need to pass and integer as the table id")
    return _cmor.set_table(table)


def close(var_id=None, file_name=False, preserve=False):
    """ Close CMOR variables/file
    Usage:
      cmor.close(varid=None)
    Where:
      var_id: id of variable to close, if passing None, means close every open ones.
      [file_name] True/False (default False) if True: return name of the file just closed, works only if var_id is not None
      [preserve] True/False (default False) if True: close the file but preserve the var definition in CMOR to write more data with this variable (into a new file)
      """
    if var_id is not None and not isinstance(var_id, int):
        raise Exception("Error var_id must be None or a integer")

    if (preserve is False):
        if (file_name is False):
            return _cmor.close(var_id, 0, 0)
        else:
            return _cmor.close(var_id, 1, 0)
    else:
        if (file_name is False):
            return _cmor.close(var_id, 0, 1)
        else:
            return _cmor.close(var_id, 1, 1)


def set_cur_dataset_attribute(name, value):
    """Sets an attribute onto the current cmor dataset
    Usage:
      cmor.set_cur_dataset_attribute(name,value)
    Where:
      name: is the name of the attribute
      value: is the value for this attribute
    """
    if value is None:
        val = ""
    else:
        val = str(value)
    return _cmor.set_cur_dataset_attribute(name, val)


def has_cur_dataset_attribute(name):
    """determines if the current cmor dataset has an attribute
    Usage:
      cmor.het_cur_dataset_attribute(name)
    Where:
      name: is the name of the attribute
    Returns True if the dataset has the attribute, False otherwise
    """
    test = _cmor.has_cur_dataset_attribute(name)
    if test == 0:
        return True
    else:
        return False


def get_cur_dataset_attribute(name):
    """Gets an attribute from the current cmor dataset
    Usage:
      cmor.get_cur_dataset_attribute(name)
    Where:
      name: is the name of the attribute
    Returns none if attribute is non-existant
    """
    if has_cur_dataset_attribute(name):
        return _cmor.get_cur_dataset_attribute(name)
    else:
        return None


def set_furtherinfourl(varid):
    """Sets further_url_info attribute for ES-DOC
    Usage:
      cmor.set_futherurlinfo(var_id)
    Where:
      var_id: is cmor variable id
    """
    return _cmor.set_furtherinfourl(varid)

def set_variable_attribute(var_id, name, data_type, value):
    """Sets an attribute onto a cmor variable
    Usage:
      cmor.set_variable_attribute(var_id,name,value)
    Where:
      var_id: is cmor variable id
      name  : is the name of the attribute
      data_type  : is the data type of the attribute
      value : is the value for this attribute
    """
    return _cmor.set_variable_attribute(var_id, name, data_type, value)


def set_deflate(var_id, shuffle, deflate, deflate_level):
    """Sets shuffle/deflate on a cmor variable
    Usage:
      cmor.set_deflate(var_id, shuffle, deflate, deflate_level)
    Where:
      var_id: is cmor variable id
      shuffle: if true, turn on netCDF the shuffle filter
      deflate: if true, turn on the deflate filter at the level
               specified by the deflate_level parameter
      deflate_level: if the deflate parameter is non-zero.
                     Set the deflate value. Must be between 0 and 9

    """

    return _cmor.set_deflate(var_id, shuffle, deflate, deflate_level)


def set_zstandard(var_id, zstandar_level):
    """Sets Zstandard compression on a cmor variable
    Usage:
      cmor.set_zstandard(var_id, szstandar_level)
    Where:
      var_id: is cmor variable id
      zstandar_level: Compression level. Must be set from -131072 to 22

    """

    return _cmor.set_zstandard(var_id, zstandar_level)


def set_quantize(var_id, quantize_mode, quantize_nsd):
    """Sets quantization on a cmor variable
    Usage:
      cmor.set_quantize(var_id, quantize_mode, quantize_nsd)
    Where:
      var_id: is cmor variable id
      quantize_mode: Quantization mode.  Can be set to the following.
                     0: No quantization mode
                     1: BitGroom
                     2: Granular BitRound
                     3: BitRound
      quantize_nsd: Number of significant digits.  If quantize_mode is set to
                    1 or 2, then the value can be set from 1 to 7 for floats
                    and 1 to 23 for doubles. If quantize_mode is set to 3, then
                    the value can be set from 1 to 15 for floats and 1 to 52
                    for doubles.  The value is ignore if quantize_mode is 0.

    """

    return _cmor.set_quantize(var_id, quantize_mode, quantize_nsd)


def has_variable_attribute(var_id, name):
    """determines if the a cmor variable has an attribute
    Usage:
      cmor.het_variable_attribute(name)
    Where:
      var_id: is cmor variable id
      name: is the name of the attribute
    Returns True if the dataset has the attribute, False otherwise
    """
    test = _cmor.has_variable_attribute(var_id, name)
    if test == 0:
        return True
    else:
        return False


def get_variable_attribute(var_id, name):
    """Gets an attribute from a cmor variable
    Usage:
      cmor.get_variable_attribute(name)
    Where:
      var_id: is cmor variable id
      name: is the name of the attribute
    Returns none if attribute is non-existant
    """
    # print 'In there asking for attribute: ',name,'on var',var_id
    if has_variable_attribute(var_id, name):
        # print 'Seems to have it',var_id,name
        return _cmor.get_variable_attribute(var_id, name)
    else:
        return None


def get_final_filename():
    """ Retrieve renamed file after cmor.close() has been called.  This is useful to reopen the file in the same program.
    """
    return _cmor.get_final_filename()



from __future__ import print_function
import cmor
import numpy
import cdms2

f = cdms2.open("Test/GR30s_halo.nc")

nlon = f['grid_center_lat'].shape[1]
nlat = f['grid_center_lat'].shape[0]


error_flag = cmor.setup(inpath='Test', netcdf_file_action=cmor.CMOR_APPEND)

error_flag = cmor.dataset(
    outpath='Joerg',
    experiment_id='DcppC2',
    institution='GICC (Generic International Climate Center, Geneva, Switzerland)',
    institute_id="GICC",
    source='GICCM1 (2002): ',
    calendar='360_day',
    realization=1,
    contact='Rusty Koder (koder@middle_earth.net) ',
    history='Output from archivcl_A1.nce/giccm_03_std_2xCO2_2256.',
    comment='Equilibrium reached after 30-year spin-up ',
    references='Model described by Koder and Tolkien ',
    model_id="GICCM1",
    forcing="Ant",
    parent_experiment_id="lgm", branch_time=0)

ntables = [cmor.load_table("Tables/CMIP6_grids")]
ntables.append(cmor.load_table("Tables/CMIP5_OImon"))


cmor.set_table(ntables[0])
axes = numpy.zeros(2, numpy.int32)

axes[0] = cmor.axis(
    table_entry='i_index',
    length=nlon,
    coord_vals=numpy.arange(0, nlon, 1, numpy.float32),
    units='1')

axes[1] = cmor.axis(
    table_entry='j_index',
    length=nlat,
    coord_vals=numpy.arange(0, nlat, 1, numpy.float32),
    units='1')


olat_val = f("grid_center_lat").filled().astype('f')
olon_val = f("grid_center_lon").filled().astype('f')
bnds_olat = f("grid_corner_lat").filled().astype('f')
bnds_olon = f("grid_corner_lon").filled().astype('f')

grid_id = cmor.grid(
    axis_ids=axes,
    latitude=olat_val,
    longitude=olon_val,
    latitude_vertices=bnds_olat,
    longitude_vertices=bnds_olon)

cmor.set_table(ntables[1])


ntimes = 12

tim_id = cmor.axis(table_entry="time",
                   units="months since 2010")
time_vals = numpy.arange(ntimes).astype('f')
bnds_time = numpy.arange(ntimes + 1).astype('f')

var_ids = cmor.variable(
    table_entry="sic",
    units="%",
    ##     positive           = vartabin(3,i),
    axis_ids=[grid_id, tim_id]
)

fnm = ""
for i in range(ntimes):
    print('writing time:', time_vals[i], bnds_time[i:i + 2])
    data = numpy.random.random((nlon, nlat))
    error_flag = cmor.write(
        var_id=var_ids,
        data=data,
        ntimes_passed=1,
        file_suffix=fnm,
        time_vals=time_vals[i],
        time_bnds=bnds_time[i:i + 2])

    fnm = cmor.close(var_ids, file_name=True, preserve=True)
    print('dumped to:', fnm)

cmor.close()



import os
import sys
xml_pth = os.path.join(sys.prefix, "share", "udunits", "udunits2.xml")
if os.path.exists(xml_pth):
    os.environ["UDUNITS2_XML_PATH"] = xml_pth

from cmor.cmor_const import *

from cmor.pywrapper import (
    CMORError, axis, variable, write, setup, load_table, set_table, zfactor,
    close, grid, set_grid_mapping, time_varying_grid_coordinate, dataset_json,
    set_cur_dataset_attribute, get_cur_dataset_attribute,
    has_cur_dataset_attribute, set_variable_attribute, get_variable_attribute,
    has_variable_attribute, get_final_filename, set_deflate, set_zstandard,
    set_quantize, set_furtherinfourl, set_climatology, get_climatology,
    set_terminate_signal, get_terminate_signal)

try:
    from check_CMOR_compliant import checkCMOR
except ImportError:
    pass
